-- ==============================================================
-- FILE : TOP.VHD - may be modified by the ADVANCED USER
-- ==============================================================
--
-- This file is the top level of your project when using
-- Camera Link with the HERON-FPGA9 module.
--
-- This file could be modified by ADVANCED users.
--
-- ==============================================================
--
--  Module      : TOP
--  Date        : 01/07/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : TOP level
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    01-07-04    Copy of the CameraLink
--                                     example for the HERON-FPGA7.
--                                     (example version 1.0).
--  1.2     R. Williams    01-08-05    Changes made to DDR memory
--                                     interface to increase the
--                                     performance when reading.
--  2.0     R. Williams    16-09-05    Added PowerPC to DDR memory
--                                     interface.
--  2.1     R. Williams    09-06-06    Removed HEPC8 support.
--                                     Minor fix to HE_RD_6F logic.
--                                     Processor access sequencing
--                                     fix to HE_DDR logic.
--  2.3     R. Williams    16-10-07    Added address enable to
--                                     user message interface to
--                                     improve response to bursts.
--  2.5     R. Williams    29-04-09    Changed logic controlling
--                                     the DDR read-data input FF
--                                     clock enables to improve
--                                     DDR data margins.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use work.CONFIG.all;

-- synopsys translate_off
library UNISIM;
  use UNISIM.vcomponents.all;
-- synopsys translate_on

entity TOP is
  port (
    -- PADS - HE_WCLK
    doclk         : in  std_logic;                       -- FIFO_OUT clock (use with DLL)
    f0            : out std_logic;                       -- FIFO_OUT clock loop driver
    -- PADS - HE_WR_6F
    dof_ff        : in  std_logic_vector(5 downto 0);    -- FIFO_OUT full flag
    dof_af        : in  std_logic_vector(5 downto 0);    -- FIFO_OUT almost full flag
    dof_wen       : out std_logic_vector(5 downto 0);    -- FIFO_OUT write enable
    do            : out std_logic_vector(31 downto 0);   -- FIFO_OUT data
    -- PADS - HE_RCLK
    diclk         : in  std_logic;                       -- FIFO_IN clock (use with DLL)
    f1            : out std_logic;                       -- FIFO_IN clock loop driver
    -- PADS - HE_RD_6F
    dif_ef        : in  std_logic_vector(5 downto 0);    -- FIFO_IN empty flag
    dif_ae        : in  std_logic_vector(5 downto 0);    -- FIFO_IN almost empty flag
    dif_ren       : out std_logic_vector(5 downto 0);    -- FIFO_IN read enable
    dif_oen       : out std_logic_vector(5 downto 0);    -- FIFO_IN output enable
    di            : in  std_logic_vector(31 downto 0);   -- FIFO_IN data
    -- PADS - FLASH Interface
    flash_a       : out std_logic_vector(24 downto 0);   -- FLASH Address
    flash_d       : inout std_logic_vector(7 downto 0);  -- FLASH Data
    flash_cen     : out std_logic;                       -- FLASH Chip Enable
    flash_wen     : out std_logic;                       -- FLASH Write Enable
    flash_oen     : out std_logic;                       -- FLASH Output Enable
    flash_rpn     : out std_logic;                       -- FLASH Reset/Power Down
    flash_vp      : out std_logic;                       -- FLASH Program Enable
    flash_st      : in  std_logic;                       -- FLASH Status
    -- PADS - HE_DDR
    sdram_clkin   : in  std_logic;                       -- DDR SDRAM A Clock input
    sdram_a_clk_p : out std_logic;                       -- DDR SDRAM A Clock Source
    sdram_a_clk_n : out std_logic;                       -- DDR SDRAM A Clock Source complement
    sdram_a_a     : out std_logic_vector(12 downto 0);   -- DDR SDRAM A Address
    sdram_a_ba    : out std_logic_vector(1 downto 0);    -- DDR SDRAM A Bank Address
    sdram_a_dq    : inout std_logic_vector(31 downto 0):= (OTHERS => 'Z'); -- DDR A SDRAM Data
    sdram_a_dqs   : inout std_logic_vector(3 downto 0):= (OTHERS => 'Z');  -- DDR A SDRAM Data Strobes
    sdram_a_dm    : out std_logic_vector(3 downto 0);    -- DDR SDRAM A Data mask
    sdram_a_cke   : out std_logic;                       -- DDR SDRAM A Clock Enable
    sdram_a_cs    : out std_logic;                       -- DDR SDRAM A Chip Select
    sdram_a_ras   : out std_logic;                       -- DDR SDRAM A Row address strobe
    sdram_a_cas   : out std_logic;                       -- DDR SDRAM A Col address strobe
    sdram_a_we    : out std_logic;                       -- DDR SDRAM A Write Enable
    sdram_a_dxi   : in  std_logic;                       -- DDR SDRAM A - to ensure Vref is created
    sdram_a_dxo   : out std_logic;                       -- DDR SDRAM A - to ensure Vref is created
    sdram_b_clk_p : out std_logic;                       -- DDR SDRAM B Clock Source
    sdram_b_clk_n : out std_logic;                       -- DDR SDRAM B Clock Source complement
    sdram_b_a     : out std_logic_vector(12 downto 0);   -- DDR SDRAM B Address
    sdram_b_ba    : out std_logic_vector(1 downto 0);    -- DDR SDRAM B Bank Address
    sdram_b_dq    : inout std_logic_vector(31 downto 0):= (OTHERS => 'Z'); -- DDR B SDRAM Data
    sdram_b_dqs   : inout std_logic_vector(3 downto 0):= (OTHERS => 'Z');  -- DDR B SDRAM Data Strobes
    sdram_b_dm    : out std_logic_vector(3 downto 0);    -- DDR SDRAM B Data mask
    sdram_b_cke   : out std_logic;                       -- DDR SDRAM B Clock Enable
    sdram_b_cs    : out std_logic;                       -- DDR SDRAM B Chip Select
    sdram_b_ras   : out std_logic;                       -- DDR SDRAM B Row address strobe
    sdram_b_cas   : out std_logic;                       -- DDR SDRAM B Col address strobe
    sdram_b_we    : out std_logic;                       -- DDR SDRAM B Write Enable
    sdram_b_dxi   : in  std_logic;                       -- DDR SDRAM B - to ensure Vref is created
    sdram_b_dxo   : out std_logic;                       -- DDR SDRAM B - to ensure Vref is created
    -- PADS - HSB
    vpd           : inout std_logic_vector(7 downto 0);  -- HSB data
    vinit         : inout std_logic;                     -- HSB address strobe
    vcs           : inout std_logic;                     -- HSB data strobe
    vwrite        : inout std_logic;                     -- HSB read(high)/write(low)
    vbusy         : inout std_logic;                     -- HSB ready
    -- PADS - Connector Digital I/O
    dio_a         : inout std_logic_vector(14 downto 0); -- Connector A IO
    dio_b         : inout std_logic_vector(14 downto 0); -- Connector B IO
    -- CONTROL signals
    config        : in  std_logic;                       -- config from carrier card
    reset_pin     : in  std_logic;                       -- reset from carrier card
    udpres        : out std_logic;                       -- reset the carrier (high if NOT used)
    -- OSC/CLOCKS
    osc0_pin      : in  std_logic;                       -- user oscillator from socketed Xtal Osc
    osc3_pin      : in  std_logic;                       -- user oscillator from surface mount Xtal Osc
    -- MISC I/O
    led           : out std_logic_vector(4 downto 0);    -- LEDs
    cid           : in  std_logic_vector(3 downto 0);    -- Carrier ID
    mid           : in  std_logic_vector(3 downto 0);    -- Module ID
    umi           : inout std_logic_vector(3 downto 0);  -- Uncommitted UMI interconnects
    addr_flagsel  : in  std_logic;                       -- Carrier Addr/Flagsel signal
    booten        : out std_logic                        -- Carrier Boot Enable signal
  );
end TOP;


architecture RTL of TOP is

  component IBUF
    port(I : in  std_logic;
         O : out std_logic);
  end component;

  component IBUFDS
    port(
      I  : in  std_logic;
      IB : in  std_logic;
      O  : out std_logic );
  end component;

  component OBUFDS
    port(
      I  : in  std_logic;
      O  : out std_logic;
      OB : out std_logic );
  end component;

  component IBUFGDS
    port(
      I  : in  std_logic;
      IB : in  std_logic;
      O  : out std_logic );
  end component;

  component IBUFG
    port(I : in  std_logic;
         O : out std_logic);
  end component;

  component IOBUF
    port(T  : in  std_logic;
         I  : in  std_logic;
         O  : out std_logic;
         IO : inout std_logic);
  end component;

  component OBUF
    port(I : in  std_logic;
         O : out std_logic);
  end component;

  component OBUFT
    port (I : in std_logic;
          T : in std_logic;
          O : out std_logic);
  end component;

  component HE_RWCLK
    generic ( HFreq : boolean := FALSE );
    port (
      -- PADS
      DCLK : in  std_logic;
      FCLK : out std_logic;
      -- I/O module
      RST  : in  std_logic;
      CLK  : in  std_logic;
      GCLK : out std_logic
    );
  end component;

  component HE_RD_6F
    port (
      -- PADS
      DIF_EF          : in  std_logic_vector(5 downto 0);
      DIF_AE          : in  std_logic_vector(5 downto 0);
      DIF_REN         : out std_logic_vector(5 downto 0);
      DIF_OEN         : out std_logic_vector(5 downto 0);
      DI              : in  std_logic_vector(31 downto 0);
      -- I/O module
      RST             : in  std_logic;
      CLK             : in  std_logic;
      INFIFO_READ_REQ : in  std_logic_vector(5 downto 0);
      INFIFO_DVALID   : out std_logic_vector(5 downto 0);
      INFIFO_SINGLE   : out std_logic_vector(5 downto 0);
      INFIFO_BURST    : out std_logic_vector(5 downto 0);
      INFIFO0_D       : out std_logic_vector(31 downto 0);
      INFIFO1_D       : out std_logic_vector(31 downto 0);
      INFIFO2_D       : out std_logic_vector(31 downto 0);
      INFIFO3_D       : out std_logic_vector(31 downto 0);
      INFIFO4_D       : out std_logic_vector(31 downto 0);
      INFIFO5_D       : out std_logic_vector(31 downto 0)
    );
  end component;

  component HE_WR_6F
    port (
      -- PADS
      DOF_FF        : in  std_logic_vector(5 downto 0);
      DOF_AF        : in  std_logic_vector(5 downto 0);
      DOF_WEN       : out std_logic_vector(5 downto 0);
      DO            : out std_logic_vector(31 downto 0);
      -- I/O module
      RST           : in  std_logic;
      CLK           : in  std_logic;
      OUTFIFO_WRITE : in  std_logic_vector(5 downto 0);
      OUTFIFO_D     : in  std_logic_vector(31 downto 0);
      OUTFIFO_READY : out std_logic_vector(5 downto 0)
    );
  end component;

  component HE_DDR
    port (
      -- PADS
      ddr_a_clk         : out std_logic;
      ddr_a_clkb        : out std_logic;
      ddr_a_address     : out std_logic_vector(12 downto 0);
      ddr_a_BA          : out std_logic_vector(1 downto 0);
      ddr_a_dq_in       : in  std_logic_vector(31 downto 0);
      ddr_a_dq_out      : out std_logic_vector(31 downto 0);
      ddr_a_dq_en       : out std_logic_vector(31 downto 0);
      ddr_a_dqs_in      : in  std_logic_vector(3 downto 0);
      ddr_a_dqs_out     : out std_logic_vector(3 downto 0);
      ddr_a_dqs_en      : out std_logic_vector(3 downto 0);
      ddr_a_dm_out      : out std_logic_vector(3 downto 0);
      ddr_a_dm_en       : out std_logic_vector(3 downto 0);
      ddr_a_cke         : out std_logic;
      ddr_a_csb         : out std_logic;
      ddr_a_rasb        : out std_logic;
      ddr_a_casb        : out std_logic;
      ddr_a_web         : out std_logic;
      ddr_b_clk         : out std_logic;
      ddr_b_clkb        : out std_logic;
      ddr_b_address     : out std_logic_vector(12 downto 0);
      ddr_b_BA          : out std_logic_vector(1 downto 0);
      ddr_b_dq_in       : in  std_logic_vector(31 downto 0);
      ddr_b_dq_out      : out std_logic_vector(31 downto 0);
      ddr_b_dq_en       : out std_logic_vector(31 downto 0);
      ddr_b_dqs_in      : in  std_logic_vector(3 downto 0);
      ddr_b_dqs_out     : out std_logic_vector(3 downto 0);
      ddr_b_dqs_en      : out std_logic_vector(3 downto 0);
      ddr_b_dm_out      : out std_logic_vector(3 downto 0);
      ddr_b_dm_en       : out std_logic_vector(3 downto 0);
      ddr_b_cke         : out std_logic;
      ddr_b_csb         : out std_logic;
      ddr_b_rasb        : out std_logic;
      ddr_b_casb        : out std_logic;
      ddr_b_web         : out std_logic;
      -- Module I/Os
      user_input_clk    : in  std_logic;
      user_rst          : in  std_logic;
      -- DDR User Clock A
      DDR_A_USER_CLK    : in  std_logic;
      -- DDR User Reset A
      DDR_A_USER_RST    : in  std_logic;
      -- DDR Data Count
      DDR_A_DATA_COUNT  : out std_logic_vector(31 downto 0);
      -- DDR Write Port A
      DDR_A_WR_DATA     : in  std_logic_vector(71 downto 0);
      DDR_A_WR_ADDR     : in  std_logic_vector(24 downto 0);
      DDR_A_WR_ADDR_WEN : in  std_logic;
      DDR_A_WR_ADDR_FF  : out std_logic;
      DDR_A_WR_ADDR_AF  : out std_logic;
      DDR_A_WR_RISE_WEN : in  std_logic;
      DDR_A_WR_RISE_FF  : out std_logic;
      DDR_A_WR_RISE_AF  : out std_logic;
      DDR_A_WR_FALL_WEN : in  std_logic;
      DDR_A_WR_FALL_FF  : out std_logic;
      DDR_A_WR_FALL_AF  : out std_logic;
      DDR_A_WR_DM_WEN   : in  std_logic;
      DDR_A_WR_DM_FF    : out std_logic;
      DDR_A_WR_DM_AF    : out std_logic;
      -- DDR Read Port A
      DDR_A_RD_DATA     : out std_logic_vector(63 downto 0);
      DDR_A_RD_ADDR     : in  std_logic_vector(24 downto 0);
      DDR_A_RD_ADDR_WEN : in  std_logic;
      DDR_A_RD_ADDR_FF  : out std_logic;
      DDR_A_RD_ADDR_AF  : out std_logic;
      DDR_A_RD_RISE_REN : in  std_logic;
      DDR_A_RD_RISE_EF  : out std_logic;
      DDR_A_RD_RISE_AE  : out std_logic;
      DDR_A_RD_FALL_REN : in  std_logic;
      DDR_A_RD_FALL_EF  : out std_logic;
      DDR_A_RD_FALL_AE  : out std_logic;
      -- DDR User Clock B
      DDR_B_USER_CLK    : in  std_logic;
      -- DDR User Reset B
      DDR_B_USER_RST    : in  std_logic;
      -- DDR Data Count
      DDR_B_DATA_COUNT  : out std_logic_vector(31 downto 0);
      -- DDR Write Port B
      DDR_B_WR_DATA     : in  std_logic_vector(71 downto 0);
      DDR_B_WR_ADDR     : in  std_logic_vector(24 downto 0);
      DDR_B_WR_ADDR_WEN : in  std_logic;
      DDR_B_WR_ADDR_FF  : out std_logic;
      DDR_B_WR_ADDR_AF  : out std_logic;
      DDR_B_WR_RISE_WEN : in  std_logic;
      DDR_B_WR_RISE_FF  : out std_logic;
      DDR_B_WR_RISE_AF  : out std_logic;
      DDR_B_WR_FALL_WEN : in  std_logic;
      DDR_B_WR_FALL_FF  : out std_logic;
      DDR_B_WR_FALL_AF  : out std_logic;
      DDR_B_WR_DM_WEN   : in  std_logic;
      DDR_B_WR_DM_FF    : out std_logic;
      DDR_B_WR_DM_AF    : out std_logic;
      -- DDR Read Port B
      DDR_B_RD_DATA     : out std_logic_vector(63 downto 0);
      DDR_B_RD_ADDR     : in  std_logic_vector(24 downto 0);
      DDR_B_RD_ADDR_WEN : in  std_logic;
      DDR_B_RD_ADDR_FF  : out std_logic;
      DDR_B_RD_ADDR_AF  : out std_logic;
      DDR_B_RD_RISE_REN : in  std_logic;
      DDR_B_RD_RISE_EF  : out std_logic;
      DDR_B_RD_RISE_AE  : out std_logic;
      DDR_B_RD_FALL_REN : in  std_logic;
      DDR_B_RD_FALL_EF  : out std_logic;
      DDR_B_RD_FALL_AE  : out std_logic;
      -- DDR PPC Interface for Port A
      PPC_TO_DDR_IF_A   : in  std_logic_vector(130 downto 0);
      DDR_TO_PPC_IF_A   : out std_logic_vector(77 downto 0);
      -- DDR PPC Interface for Port B
      PPC_TO_DDR_IF_B   : in  std_logic_vector(130 downto 0);
      DDR_TO_PPC_IF_B   : out std_logic_vector(77 downto 0)
    );
  end component;

  component HE_USER
    port (
      -- PADS - HE_V/S_CTL
      VPD_IN     : in  std_logic_vector(7 downto 0);
      VPD_OUT    : out std_logic_vector(7 downto 0);
      VPD_EN     : out std_logic;
      VINIT_IN   : in  std_logic;
      VINIT_OUT  : out std_logic;
      VINIT_EN   : out std_logic;
      VCS_IN     : in  std_logic;
      VCS_OUT    : out std_logic;
      VCS_EN     : out std_logic;
      VWRITE_IN  : in  std_logic;
      VWRITE_OUT : out std_logic;
      VWRITE_EN  : out std_logic;
      VBUSY_IN   : in  std_logic;
      VBUSY_OUT  : out std_logic;
      VBUSY_EN   : out std_logic;
      -- I/O module
      RST        : in  std_logic;
      CLK        : in  std_logic;
      READY      : in  std_logic;
      SEND_MSG   : in  std_logic;
      CE         : in  std_logic;
      DOUT       : in  std_logic_vector(7 downto 0);
      CID        : in  std_logic_vector(3 downto 0);
      MID        : in  std_logic_vector(2 downto 0);
      SEND_ID    : in  std_logic;
      LAST_BYTE  : in  std_logic;
      DIN        : out std_logic_vector(7 downto 0);
      ADDR       : out std_logic_vector(7 downto 0);
      AEN        : out std_logic;
      WEN        : out std_logic;
      REN        : out std_logic;
      DONE       : out std_logic;
      COUNT      : out std_logic;
      CLEAR      : out std_logic
    );
  end component;

  component USER_AP
    port (
      RESET             : in  std_logic;
      CONFIG            : in  std_logic;
  -- CLOCK SIGNALS
      OSC0              : in  std_logic;
      OSC3              : in  std_logic;
  -- GENERAL IO
      UDPRES            : out std_logic;
      LED               : out std_logic_vector(4 downto 0);
      ADDR_FLAGSEL      : in  std_logic;
      BOOTEN            : out std_logic;
      CID               : in  std_logic_vector(3 downto 0);
      MID               : in  std_logic_vector(3 downto 0);
  -- UMIs
      UMI_IN            : in  std_logic_vector(3 downto 0);
      UMI_OUT           : out std_logic_vector(3 downto 0);
      UMI_EN            : out std_logic_vector(3 downto 0);
  -- READ FIFO CLK
      FCLK_RD           : in  std_logic;
      SRC_FCLK_RD       : out std_logic;
  -- WRITE FIFO CLK
      FCLK_WR           : in  std_logic;
      SRC_FCLK_WR       : out std_logic;
  -- GLOBAL FIFO CLK
      FCLK_G            : in  std_logic;
      SRC_FCLK_G        : out std_logic;
  -- FIFO IN / HE_RD_6F interface
      INFIFO_READ_REQ   : out std_logic_vector(5 downto 0);
      INFIFO_DVALID     : in  std_logic_vector(5 downto 0);
      INFIFO_SINGLE     : in  std_logic_vector(5 downto 0);
      INFIFO_BURST      : in  std_logic_vector(5 downto 0);
      INFIFO0_D         : in  std_logic_vector(31 downto 0);
      INFIFO1_D         : in  std_logic_vector(31 downto 0);
      INFIFO2_D         : in  std_logic_vector(31 downto 0);
      INFIFO3_D         : in  std_logic_vector(31 downto 0);
      INFIFO4_D         : in  std_logic_vector(31 downto 0);
      INFIFO5_D         : in  std_logic_vector(31 downto 0);
  -- FIFO OUT / HE_WR_6F interface
      OUTFIFO_READY     : in  std_logic_vector(5 downto 0);
      OUTFIFO_WRITE     : out std_logic_vector(5 downto 0);
      OUTFIFO_D         : out std_logic_vector(31 downto 0);
  -- SERIAL BUS / HE_USER interface
      MSG_CLK           : out std_logic;
      MSG_DIN           : in  std_logic_vector(7 downto 0);
      MSG_ADDR          : in  std_logic_vector(7 downto 0);
      MSG_AEN           : in  std_logic;
      MSG_WEN           : in  std_logic;
      MSG_REN           : in  std_logic;
      MSG_DONE          : in  std_logic;
      MSG_COUNT         : in  std_logic;
      MSG_CLEAR         : in  std_logic;
      MSG_READY         : out std_logic;
      MSG_SEND_MSG      : out std_logic;
      MSG_CE            : out std_logic;
      MSG_DOUT          : out std_logic_vector(7 downto 0);
      MSG_SEND_ID       : out std_logic;
      MSG_LAST_BYTE     : out std_logic;
  -- CONNECTOR DIGITAL IO
      CAM_CLK           : in  std_logic;
      CAM_DATA          : in  std_logic_vector(3  downto 0);
      CAM_CTRL          : out std_logic_vector(3  downto 0);
      CAM_SER_TC        : out std_logic;
      CAM_SER_TFG       : in  std_logic;
      CONN_A_IN         : in  std_logic_vector(14 downto 10);
      CONN_A_OUT        : out std_logic_vector(14 downto 10);
      CONN_A_EN         : out std_logic_vector(14 downto 10);
      CONN_B_IN         : in  std_logic_vector(14 downto 12);
      CONN_B_OUT        : out std_logic_vector(14 downto 12);
      CONN_B_EN         : out std_logic_vector(14 downto 12);
  -- FLASH interface
      FLASH_ADDR        : out std_logic_vector(23 downto 0);
      FLASH_IN          : in  std_logic_vector(7 downto 0);
      FLASH_OUT         : out std_logic_vector(7 downto 0);
      FLASH_EN          : out std_logic_vector(7 downto 0);
      FLASH_CE          : out std_logic;
      FLASH_WE          : out std_logic;
      FLASH_OE          : out std_logic;
      FLASH_RP          : out std_logic;
      FLASH_VPEN        : out std_logic;
      FLASH_STS         : in  std_logic;
  -- DDR User Clock A
      DDR_A_USER_CLK    : out std_logic;
  -- DDR User Reset A
      DDR_A_USER_RST    : out std_logic;
  -- DDR Data Count
      DDR_A_DATA_COUNT  : in  std_logic_vector(31 downto 0);
  -- DDR Write Port A
      DDR_A_WR_DATA     : out std_logic_vector(71 downto 0);
      DDR_A_WR_ADDR     : out std_logic_vector(24 downto 0);
      DDR_A_WR_ADDR_WEN : out std_logic;
      DDR_A_WR_ADDR_FF  : in  std_logic;
      DDR_A_WR_ADDR_AF  : in  std_logic;
      DDR_A_WR_RISE_WEN : out std_logic;
      DDR_A_WR_RISE_FF  : in  std_logic;
      DDR_A_WR_RISE_AF  : in  std_logic;
      DDR_A_WR_FALL_WEN : out std_logic;
      DDR_A_WR_FALL_FF  : in  std_logic;
      DDR_A_WR_FALL_AF  : in  std_logic;
      DDR_A_WR_DM_WEN   : out std_logic;
      DDR_A_WR_DM_FF    : in  std_logic;
      DDR_A_WR_DM_AF    : in  std_logic;
  -- DDR Read Port A
      DDR_A_RD_DATA     : in  std_logic_vector(63 downto 0);
      DDR_A_RD_ADDR     : out std_logic_vector(24 downto 0);
      DDR_A_RD_ADDR_WEN : out std_logic;
      DDR_A_RD_ADDR_FF  : in  std_logic;
      DDR_A_RD_ADDR_AF  : in  std_logic;
      DDR_A_RD_RISE_REN : out std_logic;
      DDR_A_RD_RISE_EF  : in  std_logic;
      DDR_A_RD_RISE_AE  : in  std_logic;
      DDR_A_RD_FALL_REN : out std_logic;
      DDR_A_RD_FALL_EF  : in  std_logic;
      DDR_A_RD_FALL_AE  : in  std_logic;
  -- DDR User Clock B
      DDR_B_USER_CLK    : out std_logic;
  -- DDR User Reset B
      DDR_B_USER_RST    : out std_logic;
  -- DDR Data Count
      DDR_B_DATA_COUNT  : in  std_logic_vector(31 downto 0);
  -- DDR Write Port B
      DDR_B_WR_DATA     : out std_logic_vector(71 downto 0);
      DDR_B_WR_ADDR     : out std_logic_vector(24 downto 0);
      DDR_B_WR_ADDR_WEN : out std_logic;
      DDR_B_WR_ADDR_FF  : in  std_logic;
      DDR_B_WR_ADDR_AF  : in  std_logic;
      DDR_B_WR_RISE_WEN : out std_logic;
      DDR_B_WR_RISE_FF  : in  std_logic;
      DDR_B_WR_RISE_AF  : in  std_logic;
      DDR_B_WR_FALL_WEN : out std_logic;
      DDR_B_WR_FALL_FF  : in  std_logic;
      DDR_B_WR_FALL_AF  : in  std_logic;
      DDR_B_WR_DM_WEN   : out std_logic;
      DDR_B_WR_DM_FF    : in  std_logic;
      DDR_B_WR_DM_AF    : in  std_logic;
  -- DDR Read Port B
      DDR_B_RD_DATA     : in  std_logic_vector(63 downto 0);
      DDR_B_RD_ADDR     : out std_logic_vector(24 downto 0);
      DDR_B_RD_ADDR_WEN : out std_logic;
      DDR_B_RD_ADDR_FF  : in  std_logic;
      DDR_B_RD_ADDR_AF  : in  std_logic;
      DDR_B_RD_RISE_REN : out std_logic;
      DDR_B_RD_RISE_EF  : in  std_logic;
      DDR_B_RD_RISE_AE  : in  std_logic;
      DDR_B_RD_FALL_REN : out std_logic;
      DDR_B_RD_FALL_EF  : in  std_logic;
      DDR_B_RD_FALL_AE  : in  std_logic;
  -- DDR PPC Interface for Port A
      PPC_TO_DDR_IF_A   : out std_logic_vector(130 downto 0);
      DDR_TO_PPC_IF_A   : in  std_logic_vector(77 downto 0);
  -- DDR PPC Interface for Port B
      PPC_TO_DDR_IF_B   : out std_logic_vector(130 downto 0);
      DDR_TO_PPC_IF_B   : in  std_logic_vector(77 downto 0)
    );
  end component;

  attribute IOSTANDARD : string;
  attribute DRIVE      : string;
  attribute FAST       : string;

  -- Write Clock and Read Clock Buffer Definitions
  attribute FAST of oF0  : label is "TRUE";
  attribute FAST of oF1  : label is "TRUE";

  attribute DRIVE of oF0 : label is "24";
  attribute DRIVE of oF1 : label is "24";

  attribute IOSTANDARD of oF0    : label is "LVTTL";
  attribute IOSTANDARD of oF1    : label is "LVTTL";

  attribute IOSTANDARD of iDICLK : label is "LVTTL";
  attribute IOSTANDARD of iDOCLK : label is "LVTTL";

  -- Write FIFO Interface Buffer Definitions
  attribute IOSTANDARD of oDO0  : label is "LVTTL";
  attribute IOSTANDARD of oDO1  : label is "LVTTL";
  attribute IOSTANDARD of oDO2  : label is "LVTTL";
  attribute IOSTANDARD of oDO3  : label is "LVTTL";
  attribute IOSTANDARD of oDO4  : label is "LVTTL";
  attribute IOSTANDARD of oDO5  : label is "LVTTL";
  attribute IOSTANDARD of oDO6  : label is "LVTTL";
  attribute IOSTANDARD of oDO7  : label is "LVTTL";
  attribute IOSTANDARD of oDO8  : label is "LVTTL";
  attribute IOSTANDARD of oDO9  : label is "LVTTL";
  attribute IOSTANDARD of oDO10 : label is "LVTTL";
  attribute IOSTANDARD of oDO11 : label is "LVTTL";
  attribute IOSTANDARD of oDO12 : label is "LVTTL";
  attribute IOSTANDARD of oDO13 : label is "LVTTL";
  attribute IOSTANDARD of oDO14 : label is "LVTTL";
  attribute IOSTANDARD of oDO15 : label is "LVTTL";
  attribute IOSTANDARD of oDO16 : label is "LVTTL";
  attribute IOSTANDARD of oDO17 : label is "LVTTL";
  attribute IOSTANDARD of oDO18 : label is "LVTTL";
  attribute IOSTANDARD of oDO19 : label is "LVTTL";
  attribute IOSTANDARD of oDO20 : label is "LVTTL";
  attribute IOSTANDARD of oDO21 : label is "LVTTL";
  attribute IOSTANDARD of oDO22 : label is "LVTTL";
  attribute IOSTANDARD of oDO23 : label is "LVTTL";
  attribute IOSTANDARD of oDO24 : label is "LVTTL";
  attribute IOSTANDARD of oDO25 : label is "LVTTL";
  attribute IOSTANDARD of oDO26 : label is "LVTTL";
  attribute IOSTANDARD of oDO27 : label is "LVTTL";
  attribute IOSTANDARD of oDO28 : label is "LVTTL";
  attribute IOSTANDARD of oDO29 : label is "LVTTL";
  attribute IOSTANDARD of oDO30 : label is "LVTTL";
  attribute IOSTANDARD of oDO31 : label is "LVTTL";

  attribute IOSTANDARD of iFF0 : label is "LVTTL";
  attribute IOSTANDARD of iFF1 : label is "LVTTL";
  attribute IOSTANDARD of iFF2 : label is "LVTTL";
  attribute IOSTANDARD of iFF3 : label is "LVTTL";
  attribute IOSTANDARD of iFF4 : label is "LVTTL";
  attribute IOSTANDARD of iFF5 : label is "LVTTL";

  attribute IOSTANDARD of iAF0 : label is "LVTTL";
  attribute IOSTANDARD of iAF1 : label is "LVTTL";
  attribute IOSTANDARD of iAF2 : label is "LVTTL";
  attribute IOSTANDARD of iAF3 : label is "LVTTL";
  attribute IOSTANDARD of iAF4 : label is "LVTTL";
  attribute IOSTANDARD of iAF5 : label is "LVTTL";

  attribute IOSTANDARD of oWEN0 : label is "LVTTL";
  attribute IOSTANDARD of oWEN1 : label is "LVTTL";
  attribute IOSTANDARD of oWEN2 : label is "LVTTL";
  attribute IOSTANDARD of oWEN3 : label is "LVTTL";
  attribute IOSTANDARD of oWEN4 : label is "LVTTL";
  attribute IOSTANDARD of oWEN5 : label is "LVTTL";

  attribute FAST of oWEN0 : label is "TRUE";
  attribute FAST of oWEN1 : label is "TRUE";
  attribute FAST of oWEN2 : label is "TRUE";
  attribute FAST of oWEN3 : label is "TRUE";
  attribute FAST of oWEN4 : label is "TRUE";
  attribute FAST of oWEN5 : label is "TRUE";

  attribute DRIVE of oWEN0 : label is "8";
  attribute DRIVE of oWEN1 : label is "8";
  attribute DRIVE of oWEN2 : label is "8";
  attribute DRIVE of oWEN3 : label is "8";
  attribute DRIVE of oWEN4 : label is "8";
  attribute DRIVE of oWEN5 : label is "8";

  -- Read FIFO Interface Buffer Definitions
  attribute IOSTANDARD of iDI0  : label is "LVTTL";
  attribute IOSTANDARD of iDI1  : label is "LVTTL";
  attribute IOSTANDARD of iDI2  : label is "LVTTL";
  attribute IOSTANDARD of iDI3  : label is "LVTTL";
  attribute IOSTANDARD of iDI4  : label is "LVTTL";
  attribute IOSTANDARD of iDI5  : label is "LVTTL";
  attribute IOSTANDARD of iDI6  : label is "LVTTL";
  attribute IOSTANDARD of iDI7  : label is "LVTTL";
  attribute IOSTANDARD of iDI8  : label is "LVTTL";
  attribute IOSTANDARD of iDI9  : label is "LVTTL";
  attribute IOSTANDARD of iDI10 : label is "LVTTL";
  attribute IOSTANDARD of iDI11 : label is "LVTTL";
  attribute IOSTANDARD of iDI12 : label is "LVTTL";
  attribute IOSTANDARD of iDI13 : label is "LVTTL";
  attribute IOSTANDARD of iDI14 : label is "LVTTL";
  attribute IOSTANDARD of iDI15 : label is "LVTTL";
  attribute IOSTANDARD of iDI16 : label is "LVTTL";
  attribute IOSTANDARD of iDI17 : label is "LVTTL";
  attribute IOSTANDARD of iDI18 : label is "LVTTL";
  attribute IOSTANDARD of iDI19 : label is "LVTTL";
  attribute IOSTANDARD of iDI20 : label is "LVTTL";
  attribute IOSTANDARD of iDI21 : label is "LVTTL";
  attribute IOSTANDARD of iDI22 : label is "LVTTL";
  attribute IOSTANDARD of iDI23 : label is "LVTTL";
  attribute IOSTANDARD of iDI24 : label is "LVTTL";
  attribute IOSTANDARD of iDI25 : label is "LVTTL";
  attribute IOSTANDARD of iDI26 : label is "LVTTL";
  attribute IOSTANDARD of iDI27 : label is "LVTTL";
  attribute IOSTANDARD of iDI28 : label is "LVTTL";
  attribute IOSTANDARD of iDI29 : label is "LVTTL";
  attribute IOSTANDARD of iDI30 : label is "LVTTL";
  attribute IOSTANDARD of iDI31 : label is "LVTTL";

  attribute IOSTANDARD of iEF0 : label is "LVTTL";
  attribute IOSTANDARD of iEF1 : label is "LVTTL";
  attribute IOSTANDARD of iEF2 : label is "LVTTL";
  attribute IOSTANDARD of iEF3 : label is "LVTTL";
  attribute IOSTANDARD of iEF4 : label is "LVTTL";
  attribute IOSTANDARD of iEF5 : label is "LVTTL";

  attribute IOSTANDARD of iAE0 : label is "LVTTL";
  attribute IOSTANDARD of iAE1 : label is "LVTTL";
  attribute IOSTANDARD of iAE2 : label is "LVTTL";
  attribute IOSTANDARD of iAE3 : label is "LVTTL";
  attribute IOSTANDARD of iAE4 : label is "LVTTL";
  attribute IOSTANDARD of iAE5 : label is "LVTTL";

  attribute IOSTANDARD of oREN0 : label is "LVTTL";
  attribute IOSTANDARD of oREN1 : label is "LVTTL";
  attribute IOSTANDARD of oREN2 : label is "LVTTL";
  attribute IOSTANDARD of oREN3 : label is "LVTTL";
  attribute IOSTANDARD of oREN4 : label is "LVTTL";
  attribute IOSTANDARD of oREN5 : label is "LVTTL";

  attribute IOSTANDARD of oOEN0 : label is "LVTTL";
  attribute IOSTANDARD of oOEN1 : label is "LVTTL";
  attribute IOSTANDARD of oOEN2 : label is "LVTTL";
  attribute IOSTANDARD of oOEN3 : label is "LVTTL";
  attribute IOSTANDARD of oOEN4 : label is "LVTTL";
  attribute IOSTANDARD of oOEN5 : label is "LVTTL";

  attribute FAST of oREN0 : label is "TRUE";
  attribute FAST of oREN1 : label is "TRUE";
  attribute FAST of oREN2 : label is "TRUE";
  attribute FAST of oREN3 : label is "TRUE";
  attribute FAST of oREN4 : label is "TRUE";
  attribute FAST of oREN5 : label is "TRUE";

  attribute FAST of oOEN0 : label is "TRUE";
  attribute FAST of oOEN1 : label is "TRUE";
  attribute FAST of oOEN2 : label is "TRUE";
  attribute FAST of oOEN3 : label is "TRUE";
  attribute FAST of oOEN4 : label is "TRUE";
  attribute FAST of oOEN5 : label is "TRUE";

  attribute DRIVE of oREN0 : label is "8";
  attribute DRIVE of oREN1 : label is "8";
  attribute DRIVE of oREN2 : label is "8";
  attribute DRIVE of oREN3 : label is "8";
  attribute DRIVE of oREN4 : label is "8";
  attribute DRIVE of oREN5 : label is "8";

  attribute DRIVE of oOEN0 : label is "8";
  attribute DRIVE of oOEN1 : label is "8";
  attribute DRIVE of oOEN2 : label is "8";
  attribute DRIVE of oOEN3 : label is "8";
  attribute DRIVE of oOEN4 : label is "8";
  attribute DRIVE of oOEN5 : label is "8";

  -- FLASH Interface Buffer Definitions
  attribute IOSTANDARD of oFL_A0  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A1  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A2  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A3  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A4  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A5  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A6  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A7  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A8  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A9  : label is "LVTTL";
  attribute IOSTANDARD of oFL_A10 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A11 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A12 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A13 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A14 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A15 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A16 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A17 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A18 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A19 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A20 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A21 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A22 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A23 : label is "LVTTL";
  attribute IOSTANDARD of oFL_A24 : label is "LVTTL";

  attribute IOSTANDARD of iFL_D0 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D1 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D2 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D3 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D4 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D5 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D6 : label is "LVTTL";
  attribute IOSTANDARD of iFL_D7 : label is "LVTTL";

  attribute IOSTANDARD of oFL_CE : label is "LVTTL";
  attribute IOSTANDARD of oFL_WE : label is "LVTTL";
  attribute IOSTANDARD of oFL_OE : label is "LVTTL";
  attribute IOSTANDARD of oFL_RP : label is "LVTTL";
  attribute IOSTANDARD of oFL_VP : label is "LVTTL";
  attribute IOSTANDARD of iFL_ST : label is "LVTTL";

  -- DDR SDRAM Interface Buffer Definitions
  attribute IOSTANDARD of iSCLKI : label is "LVCMOS25";

  attribute IOSTANDARD of oSCLK_A0 : label is "SSTL2_I";
  attribute IOSTANDARD of oSCLK_A1 : label is "SSTL2_I";

  attribute IOSTANDARD of oSCLK_B0 : label is "SSTL2_I";
  attribute IOSTANDARD of oSCLK_B1 : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_A_AD0  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD1  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD2  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD3  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD4  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD5  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD6  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD7  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD8  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD9  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD10 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD11 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_AD12 : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_B_AD0  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD1  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD2  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD3  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD4  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD5  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD6  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD7  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD8  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD9  : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD10 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD11 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_AD12 : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_A_BA0 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_BA1 : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_B_BA0 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_BA1 : label is "SSTL2_I";

  attribute IOSTANDARD of iSD_A_DXI : label is "SSTL2_II";
  attribute IOSTANDARD of oSD_A_DXO : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_B_DXI : label is "SSTL2_II";
  attribute IOSTANDARD of oSD_B_DXO : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_A_D0  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D1  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D2  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D3  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D4  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D5  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D6  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D7  : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_A_D8  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D9  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D10 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D11 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D12 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D13 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D14 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D15 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_A_D16 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D17 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D18 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D19 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D20 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D21 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D22 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D23 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_A_D24 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D25 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D26 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D27 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D28 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D29 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D30 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_D31 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_B_D0  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D1  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D2  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D3  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D4  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D5  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D6  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D7  : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_B_D8  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D9  : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D10 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D11 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D12 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D13 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D14 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D15 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_B_D16 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D17 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D18 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D19 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D20 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D21 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D22 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D23 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_B_D24 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D25 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D26 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D27 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D28 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D29 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D30 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_D31 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_A_DQS0 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_DQS1 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_DQS2 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_A_DQS3 : label is "SSTL2_II";

  attribute IOSTANDARD of iSD_B_DQS0 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_DQS1 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_DQS2 : label is "SSTL2_II";
  attribute IOSTANDARD of iSD_B_DQS3 : label is "SSTL2_II";

  attribute IOSTANDARD of oSD_A_DM0 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_DM1 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_DM2 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_DM3 : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_B_DM0 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_DM1 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_DM2 : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_DM3 : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_A_CKE : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_CS  : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_A_RAS : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_CAS : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_A_WE  : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_B_CKE : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_CS  : label is "SSTL2_I";

  attribute IOSTANDARD of oSD_B_RAS : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_CAS : label is "SSTL2_I";
  attribute IOSTANDARD of oSD_B_WE  : label is "SSTL2_I";

  -- HSB/User Interface Buffer Definitions
  attribute IOSTANDARD of iVINI : label is "LVCMOS25";
  attribute IOSTANDARD of iVBSY : label is "LVCMOS25";

  attribute IOSTANDARD of iVPD0 : label is "LVCMOS25";
  attribute IOSTANDARD of iVPD1 : label is "LVCMOS25";
  attribute IOSTANDARD of iVPD2 : label is "LVCMOS25";
  attribute IOSTANDARD of iVPD3 : label is "LVCMOS25";

  attribute IOSTANDARD of iVPD4 : label is "LVCMOS25";
  attribute IOSTANDARD of iVPD5 : label is "LVCMOS25";
  attribute IOSTANDARD of iVPD6 : label is "LVCMOS25";
  attribute IOSTANDARD of iVPD7 : label is "LVCMOS25";

  attribute IOSTANDARD of iVCS  : label is "LVCMOS25";
  attribute IOSTANDARD of iVWR  : label is "LVCMOS25";

  -- Digital I/O Buffer Definitions
  attribute IOSTANDARD of iDIOA10 : label is "LVCMOS25";
  attribute IOSTANDARD of iDIOA11 : label is "LVCMOS25";
  attribute IOSTANDARD of iDIOA12 : label is "LVCMOS25";
  attribute IOSTANDARD of iDIOA13 : label is "LVCMOS25";
  attribute IOSTANDARD of iDIOA14 : label is "LVCMOS25";

  attribute IOSTANDARD of iDIOB12 : label is "LVCMOS25";
  attribute IOSTANDARD of iDIOB13 : label is "LVCMOS25";
  attribute IOSTANDARD of iDIOB14 : label is "LVCMOS25";

  -- Control Signal Buffer Definitions
  attribute IOSTANDARD of iCFG : label is "LVTTL";
  attribute IOSTANDARD of iRES : label is "LVTTL";
  attribute IOSTANDARD of oUDP : label is "LVTTL";

  -- OSC/Clocks Buffer Definitions
  attribute IOSTANDARD of iOSC0 : label is "LVTTL";
  attribute IOSTANDARD of iOSC3 : label is "LVTTL";

  -- Misc. I/O Buffer Definitions
  attribute IOSTANDARD of oLED0 : label is "LVTTL";
  attribute IOSTANDARD of oLED1 : label is "LVTTL";
  attribute IOSTANDARD of oLED2 : label is "LVTTL";
  attribute IOSTANDARD of oLED3 : label is "LVTTL";
  attribute IOSTANDARD of oLED4 : label is "LVTTL";

  attribute IOSTANDARD of iCID0 : label is "LVCMOS25";
  attribute IOSTANDARD of iCID1 : label is "LVCMOS25";
  attribute IOSTANDARD of iCID2 : label is "LVCMOS25";
  attribute IOSTANDARD of iCID3 : label is "LVCMOS25";

  attribute IOSTANDARD of iMID0 : label is "LVCMOS25";
  attribute IOSTANDARD of iMID1 : label is "LVCMOS25";
  attribute IOSTANDARD of iMID2 : label is "LVCMOS25";
  attribute IOSTANDARD of iMID3 : label is "LVCMOS25";

  attribute IOSTANDARD of iUMI0 : label is "LVTTL";
  attribute IOSTANDARD of iUMI1 : label is "LVTTL";
  attribute IOSTANDARD of iUMI2 : label is "LVTTL";
  attribute IOSTANDARD of iUMI3 : label is "LVTTL";

  attribute IOSTANDARD of iAFS : label is "LVTTL";
  attribute IOSTANDARD of oBEN : label is "LVTTL";

  -- Camera Link Buffer Definitions
  attribute IOSTANDARD of lvds_ibufg : label is "LVDS_25";

  attribute IOSTANDARD of lvds_ibuf0 : label is "LVDS_25";
  attribute IOSTANDARD of lvds_ibuf1 : label is "LVDS_25";
  attribute IOSTANDARD of lvds_ibuf2 : label is "LVDS_25";
  attribute IOSTANDARD of lvds_ibuf3 : label is "LVDS_25";

  attribute IOSTANDARD of lvds_obuf0 : label is "LVDS_25";
  attribute IOSTANDARD of lvds_obuf1 : label is "LVDS_25";
  attribute IOSTANDARD of lvds_obuf2 : label is "LVDS_25";
  attribute IOSTANDARD of lvds_obuf3 : label is "LVDS_25";

  attribute IOSTANDARD of lvds_obuf4 : label is "LVDS_25";

  attribute IOSTANDARD of lvds_ibuf4 : label is "LVDS_25";

  signal xLED              : std_logic_vector(4 downto 0);
  signal xCID              : std_logic_vector(3 downto 0);
  signal xMID              : std_logic_vector(3 downto 0);
  signal xRESET            : std_logic;
  signal xCONFIG           : std_logic;
  signal xUDPRES           : std_logic;
  signal xADDR_FLAGSEL     : std_logic;
  signal xBOOTEN           : std_logic;

  signal RESET             : std_logic;

  signal OSC0              : std_logic;
  signal OSC3              : std_logic;

  signal FCLK_RD           : std_logic;
  signal SRC_FCLK_RD       : std_logic;
  signal FCLK_WR           : std_logic;
  signal SRC_FCLK_WR       : std_logic;
  signal FCLK_G            : std_logic;
  signal xFCLK_G           : std_logic;
  signal SRC_FCLK_G        : std_logic;
  signal xDICLK            : std_logic;
  signal xF1               : std_logic;
  signal xDOCLK            : std_logic;
  signal xF0               : std_logic;

  signal xDIF_EF           : std_logic_vector(5 downto 0);
  signal xDIF_AE           : std_logic_vector(5 downto 0);
  signal xDIF_REN          : std_logic_vector(5 downto 0);
  signal xDIF_OEN          : std_logic_vector(5 downto 0);
  signal xDI               : std_logic_vector(31 downto 0);
  signal INFIFO_READ_REQ   : std_logic_vector(5 downto 0);
  signal INFIFO_DVALID     : std_logic_vector(5 downto 0);
  signal INFIFO_SINGLE     : std_logic_vector(5 downto 0);
  signal INFIFO_BURST      : std_logic_vector(5 downto 0);
  signal INFIFO0_D         : std_logic_vector(31 downto 0);
  signal INFIFO1_D         : std_logic_vector(31 downto 0);
  signal INFIFO2_D         : std_logic_vector(31 downto 0);
  signal INFIFO3_D         : std_logic_vector(31 downto 0);
  signal INFIFO4_D         : std_logic_vector(31 downto 0);
  signal INFIFO5_D         : std_logic_vector(31 downto 0);

  signal xDOF_FF           : std_logic_vector(5 downto 0);
  signal xDOF_AF           : std_logic_vector(5 downto 0);
  signal xDOF_WEN          : std_logic_vector(5 downto 0);
  signal xDO               : std_logic_vector(31 downto 0);
  signal OUTFIFO_READY     : std_logic_vector(5 downto 0);
  signal OUTFIFO_WRITE     : std_logic_vector(5 downto 0);
  signal OUTFIFO_D         : std_logic_vector(31 downto 0);

  signal VPD_IN            : std_logic_vector(7 downto 0);
  signal VPD_OUT           : std_logic_vector(7 downto 0);
  signal VPD_EN            : std_logic;

  signal VINIT_IN          : std_logic;
  signal VINIT_OUT         : std_logic;
  signal VINIT_EN          : std_logic;

  signal VCS_IN            : std_logic;
  signal VCS_OUT           : std_logic;
  signal VCS_EN            : std_logic;

  signal VWRITE_IN         : std_logic;
  signal VWRITE_OUT        : std_logic;
  signal VWRITE_EN         : std_logic;

  signal VBUSY_IN          : std_logic;
  signal VBUSY_OUT         : std_logic;
  signal VBUSY_EN          : std_logic;

  signal MSG_CLK           : std_logic;
  signal MSG_READY         : std_logic;
  signal MSG_SEND_MSG      : std_logic;
  signal MSG_CE            : std_logic;
  signal MSG_DOUT          : std_logic_vector(7 downto 0);
  signal MSG_SEND_ID       : std_logic;
  signal MSG_LAST_BYTE     : std_logic;
  signal MSG_DIN           : std_logic_vector(7 downto 0);
  signal MSG_ADDR          : std_logic_vector(7 downto 0);
  signal MSG_AEN           : std_logic;
  signal MSG_WEN           : std_logic;
  signal MSG_REN           : std_logic;
  signal MSG_DONE          : std_logic;
  signal MSG_COUNT         : std_logic;
  signal MSG_CLEAR         : std_logic;

  signal UMI_IN            : std_logic_vector(3 downto 0);
  signal UMI_OUT           : std_logic_vector(3 downto 0);
  signal UMI_EN            : std_logic_vector(3 downto 0);

  signal CAM_CLK           : std_logic;
  signal CAM_DATA          : std_logic_vector(3 downto 0);
  signal CAM_CTRL          : std_logic_vector(3 downto 0);
  signal CAM_SER_TC        : std_logic;
  signal CAM_SER_TFG       : std_logic;

  -- NOTE : Signals 0 to 9 on Connector A are used for Camera Link
  signal CONN_A_IN         : std_logic_vector(14 downto 10);
  signal CONN_A_OUT        : std_logic_vector(14 downto 10);
  signal CONN_A_EN         : std_logic_vector(14 downto 10);

  -- NOTE : Signals 0 to 11 on Connector B are used for Camera Link
  signal CONN_B_IN         : std_logic_vector(14 downto 12);
  signal CONN_B_OUT        : std_logic_vector(14 downto 12);
  signal CONN_B_EN         : std_logic_vector(14 downto 12);

  signal FLASH_ADDR        : std_logic_vector(24 downto 0);
  signal FLASH_IN          : std_logic_vector(7 downto 0);
  signal FLASH_OUT         : std_logic_vector(7 downto 0);
  signal FLASH_EN          : std_logic_vector(7 downto 0);
  signal FLASH_CE          : std_logic;
  signal FLASH_WE          : std_logic;
  signal FLASH_OE          : std_logic;
  signal FLASH_RP          : std_logic;
  signal FLASH_VPEN        : std_logic;
  signal FLASH_STS         : std_logic;

  signal user_input_clk    : std_logic;

  signal ddr_a_clk         : std_logic;
  signal ddr_a_clkb        : std_logic;
  signal ddr_a_address     : std_logic_vector(12 downto 0);
  signal ddr_a_BA          : std_logic_vector(1 downto 0);
  signal ddr_a_dq_in       : std_logic_vector(31 downto 0);
  signal ddr_a_dq_out      : std_logic_vector(31 downto 0);
  signal ddr_a_dq_en       : std_logic_vector(31 downto 0);
  signal ddr_a_dqs_in      : std_logic_vector(3 downto 0);
  signal ddr_a_dqs_out     : std_logic_vector(3 downto 0);
  signal ddr_a_dqs_en      : std_logic_vector(3 downto 0);
  signal ddr_a_dm_out      : std_logic_vector(3 downto 0);
  signal ddr_a_dm_en       : std_logic_vector(3 downto 0);
  signal ddr_a_cke         : std_logic;
  signal ddr_a_csb         : std_logic;
  signal ddr_a_rasb        : std_logic;
  signal ddr_a_casb        : std_logic;
  signal ddr_a_web         : std_logic;
  signal ddr_a_dx          : std_logic;

  signal ddr_b_clk         : std_logic;
  signal ddr_b_clkb        : std_logic;
  signal ddr_b_address     : std_logic_vector(12 downto 0);
  signal ddr_b_BA          : std_logic_vector(1 downto 0);
  signal ddr_b_dq_in       : std_logic_vector(31 downto 0);
  signal ddr_b_dq_out      : std_logic_vector(31 downto 0);
  signal ddr_b_dq_en       : std_logic_vector(31 downto 0);
  signal ddr_b_dqs_in      : std_logic_vector(3 downto 0);
  signal ddr_b_dqs_out     : std_logic_vector(3 downto 0);
  signal ddr_b_dqs_en      : std_logic_vector(3 downto 0);
  signal ddr_b_dm_out      : std_logic_vector(3 downto 0);
  signal ddr_b_dm_en       : std_logic_vector(3 downto 0);
  signal ddr_b_cke         : std_logic;
  signal ddr_b_csb         : std_logic;
  signal ddr_b_rasb        : std_logic;
  signal ddr_b_casb        : std_logic;
  signal ddr_b_web         : std_logic;
  signal ddr_b_dx          : std_logic;

  signal DDR_A_USER_CLK    : std_logic;
  signal DDR_B_USER_CLK    : std_logic;

  signal DDR_A_USER_RST    : std_logic;
  signal DDR_B_USER_RST    : std_logic;

  signal DDR_A_DATA_COUNT  : std_logic_vector(31 downto 0);
  signal DDR_B_DATA_COUNT  : std_logic_vector(31 downto 0);

  signal DDR_A_WR_DATA     : std_logic_vector(71 downto 0);
  signal DDR_A_WR_ADDR     : std_logic_vector(24 downto 0);
  signal DDR_A_WR_ADDR_WEN : std_logic;
  signal DDR_A_WR_ADDR_FF  : std_logic;
  signal DDR_A_WR_ADDR_AF  : std_logic;
  signal DDR_A_WR_RISE_WEN : std_logic;
  signal DDR_A_WR_RISE_FF  : std_logic;
  signal DDR_A_WR_RISE_AF  : std_logic;
  signal DDR_A_WR_FALL_WEN : std_logic;
  signal DDR_A_WR_FALL_FF  : std_logic;
  signal DDR_A_WR_FALL_AF  : std_logic;
  signal DDR_A_WR_DM_WEN   : std_logic;
  signal DDR_A_WR_DM_FF    : std_logic;
  signal DDR_A_WR_DM_AF    : std_logic;

  signal DDR_A_RD_DATA     : std_logic_vector(63 downto 0);
  signal DDR_A_RD_ADDR     : std_logic_vector(24 downto 0);
  signal DDR_A_RD_ADDR_WEN : std_logic;
  signal DDR_A_RD_ADDR_FF  : std_logic;
  signal DDR_A_RD_ADDR_AF  : std_logic;
  signal DDR_A_RD_RISE_REN : std_logic;
  signal DDR_A_RD_RISE_EF  : std_logic;
  signal DDR_A_RD_RISE_AE  : std_logic;
  signal DDR_A_RD_FALL_REN : std_logic;
  signal DDR_A_RD_FALL_EF  : std_logic;
  signal DDR_A_RD_FALL_AE  : std_logic;

  signal DDR_B_WR_DATA     : std_logic_vector(71 downto 0);
  signal DDR_B_WR_ADDR     : std_logic_vector(24 downto 0);
  signal DDR_B_WR_ADDR_WEN : std_logic;
  signal DDR_B_WR_ADDR_FF  : std_logic;
  signal DDR_B_WR_ADDR_AF  : std_logic;
  signal DDR_B_WR_RISE_WEN : std_logic;
  signal DDR_B_WR_RISE_FF  : std_logic;
  signal DDR_B_WR_RISE_AF  : std_logic;
  signal DDR_B_WR_FALL_WEN : std_logic;
  signal DDR_B_WR_FALL_FF  : std_logic;
  signal DDR_B_WR_FALL_AF  : std_logic;
  signal DDR_B_WR_DM_WEN   : std_logic;
  signal DDR_B_WR_DM_FF    : std_logic;
  signal DDR_B_WR_DM_AF    : std_logic;

  signal DDR_B_RD_DATA     : std_logic_vector(63 downto 0);
  signal DDR_B_RD_ADDR     : std_logic_vector(24 downto 0);
  signal DDR_B_RD_ADDR_WEN : std_logic;
  signal DDR_B_RD_ADDR_FF  : std_logic;
  signal DDR_B_RD_ADDR_AF  : std_logic;
  signal DDR_B_RD_RISE_REN : std_logic;
  signal DDR_B_RD_RISE_EF  : std_logic;
  signal DDR_B_RD_RISE_AE  : std_logic;
  signal DDR_B_RD_FALL_REN : std_logic;
  signal DDR_B_RD_FALL_EF  : std_logic;
  signal DDR_B_RD_FALL_AE  : std_logic;

  signal PPC_TO_DDR_IF_A   : std_logic_vector(130 downto 0);
  signal DDR_TO_PPC_IF_A   : std_logic_vector(77 downto 0);

  signal PPC_TO_DDR_IF_B   : std_logic_vector(130 downto 0);
  signal DDR_TO_PPC_IF_B   : std_logic_vector(77 downto 0);

begin

  -- RESET active low
  iRES : IBUF port map (I=>reset_pin, O=>xRESET);

  RESET <= NOT xRESET;

  -- UMIs
  iUMI0 : iobuf port map (T=>UMI_EN(0), I=>UMI_OUT(0), O=>UMI_IN(0), IO=>umi(0));
  iUMI1 : iobuf port map (T=>UMI_EN(1), I=>UMI_OUT(1), O=>UMI_IN(1), IO=>umi(1));
  iUMI2 : iobuf port map (T=>UMI_EN(2), I=>UMI_OUT(2), O=>UMI_IN(2), IO=>umi(2));
  iUMI3 : iobuf port map (T=>UMI_EN(3), I=>UMI_OUT(3), O=>UMI_IN(3), IO=>umi(3));

  -- Oscillator inputs
  iOSC0 : IBUF port map (I=>osc0_pin, O=>OSC0);
  iOSC3 : IBUF port map (I=>osc3_pin, O=>OSC3);

  -------------------------------------------------------------
  -- FIFO clocks

  iDICLK : IBUFG port map (I=>diclk, O=>xDICLK);
  oF1 : OBUF port map (I=>xF1, O=>f1);

  iDOCLK : IBUFG port map (I=>doclk, O=>xDOCLK);
  oF0 : OBUF port map (I=>xF0, O=>f0);

  -- same clock for both FIFO interfaces
  -- ===================================
  gFCKG : if (FCLK_G_DOMAIN) generate

    -- read clock
    -- ==========
    iGCLK : HE_RWCLK
      generic map (HFreq => HIGH_FCLK_G)
      port map (
        DCLK => xDICLK,
        FCLK => xF1,
        --
        RST  => RESET,
        CLK  => SRC_FCLK_G,
        GCLK => xFCLK_G );

    -- write clock
    -- ===========
    xF0 <= SRC_FCLK_G;

    -- USER_AP clocks
    FCLK_RD <= xFCLK_G;
    FCLK_WR <= xFCLK_G;
    FCLK_G  <= xFCLK_G;

  end generate gFCKG;

  -- separate clocks for RD/WR FIFOs
  -- ===============================
  gNFCKG : if (NOT FCLK_G_DOMAIN) generate

    -- read clock
    -- ==========
    iRDCLK : HE_RWCLK
      generic map (HFreq => HIGH_FCLK_RD)
      port map (
        DCLK => xDICLK,
        FCLK => xF1,
        --
        RST  => RESET,
        CLK  => SRC_FCLK_RD,
        GCLK => FCLK_RD );

    -- write clock
    -- ===========
    iWRCLK : HE_RWCLK
      generic map (HFreq => HIGH_FCLK_WR)
      port map (
        DCLK => xDOCLK,
        FCLK => xF0,
        --
        RST  => RESET,
        CLK  => SRC_FCLK_WR,
        GCLK => FCLK_WR );

    -- unused clock
    FCLK_G  <= '0';

  end generate gNFCKG;

  -------------------------------------------------------------
  -- FIFO input

  iRD : HE_RD_6F
    port map (
      DIF_EF          => xDIF_EF,
      DIF_AE          => xDIF_AE,
      DIF_REN         => xDIF_REN,
      DIF_OEN         => xDIF_OEN,
      DI              => xDI,
      --
      RST             => RESET,
      CLK             => FCLK_RD,
      INFIFO_READ_REQ => INFIFO_READ_REQ,
      INFIFO_DVALID   => INFIFO_DVALID,
      INFIFO_SINGLE   => INFIFO_SINGLE,
      INFIFO_BURST    => INFIFO_BURST,
      INFIFO0_D       => INFIFO0_D,
      INFIFO1_D       => INFIFO1_D,
      INFIFO2_D       => INFIFO2_D,
      INFIFO3_D       => INFIFO3_D,
      INFIFO4_D       => INFIFO4_D,
      INFIFO5_D       => INFIFO5_D );

  iDI0  : IBUF port map (I=>di(0),  O=>xDI(0));
  iDI1  : IBUF port map (I=>di(1),  O=>xDI(1));
  iDI2  : IBUF port map (I=>di(2),  O=>xDI(2));
  iDI3  : IBUF port map (I=>di(3),  O=>xDI(3));
  iDI4  : IBUF port map (I=>di(4),  O=>xDI(4));
  iDI5  : IBUF port map (I=>di(5),  O=>xDI(5));
  iDI6  : IBUF port map (I=>di(6),  O=>xDI(6));
  iDI7  : IBUF port map (I=>di(7),  O=>xDI(7));

  iDI8  : IBUF port map (I=>di(8),  O=>xDI(8));
  iDI9  : IBUF port map (I=>di(9),  O=>xDI(9));
  iDI10 : IBUF port map (I=>di(10), O=>xDI(10));
  iDI11 : IBUF port map (I=>di(11), O=>xDI(11));
  iDI12 : IBUF port map (I=>di(12), O=>xDI(12));
  iDI13 : IBUF port map (I=>di(13), O=>xDI(13));
  iDI14 : IBUF port map (I=>di(14), O=>xDI(14));
  iDI15 : IBUF port map (I=>di(15), O=>xDI(15));

  iDI16 : IBUF port map (I=>di(16), O=>xDI(16));
  iDI17 : IBUF port map (I=>di(17), O=>xDI(17));
  iDI18 : IBUF port map (I=>di(18), O=>xDI(18));
  iDI19 : IBUF port map (I=>di(19), O=>xDI(19));
  iDI20 : IBUF port map (I=>di(20), O=>xDI(20));
  iDI21 : IBUF port map (I=>di(21), O=>xDI(21));
  iDI22 : IBUF port map (I=>di(22), O=>xDI(22));
  iDI23 : IBUF port map (I=>di(23), O=>xDI(23));

  iDI24 : IBUF port map (I=>di(24), O=>xDI(24));
  iDI25 : IBUF port map (I=>di(25), O=>xDI(25));
  iDI26 : IBUF port map (I=>di(26), O=>xDI(26));
  iDI27 : IBUF port map (I=>di(27), O=>xDI(27));
  iDI28 : IBUF port map (I=>di(28), O=>xDI(28));
  iDI29 : IBUF port map (I=>di(29), O=>xDI(29));
  iDI30 : IBUF port map (I=>di(30), O=>xDI(30));
  iDI31 : IBUF port map (I=>di(31), O=>xDI(31));

  iEF0 : IBUF port map (I=>dif_ef(0), O=>xDIF_EF(0));
  iEF1 : IBUF port map (I=>dif_ef(1), O=>xDIF_EF(1));
  iEF2 : IBUF port map (I=>dif_ef(2), O=>xDIF_EF(2));
  iEF3 : IBUF port map (I=>dif_ef(3), O=>xDIF_EF(3));
  iEF4 : IBUF port map (I=>dif_ef(4), O=>xDIF_EF(4));
  iEF5 : IBUF port map (I=>dif_ef(5), O=>xDIF_EF(5));

  iAE0 : IBUF port map (I=>dif_ae(0), O=>xDIF_AE(0));
  iAE1 : IBUF port map (I=>dif_ae(1), O=>xDIF_AE(1));
  iAE2 : IBUF port map (I=>dif_ae(2), O=>xDIF_AE(2));
  iAE3 : IBUF port map (I=>dif_ae(3), O=>xDIF_AE(3));
  iAE4 : IBUF port map (I=>dif_ae(4), O=>xDIF_AE(4));
  iAE5 : IBUF port map (I=>dif_ae(5), O=>xDIF_AE(5));

  oREN0 : OBUF port map (I=>xDIF_REN(0), O=>dif_ren(0));
  oREN1 : OBUF port map (I=>xDIF_REN(1), O=>dif_ren(1));
  oREN2 : OBUF port map (I=>xDIF_REN(2), O=>dif_ren(2));
  oREN3 : OBUF port map (I=>xDIF_REN(3), O=>dif_ren(3));
  oREN4 : OBUF port map (I=>xDIF_REN(4), O=>dif_ren(4));
  oREN5 : OBUF port map (I=>xDIF_REN(5), O=>dif_ren(5));

  oOEN0 : OBUF port map (I=>xDIF_OEN(0), O=>dif_oen(0));
  oOEN1 : OBUF port map (I=>xDIF_OEN(1), O=>dif_oen(1));
  oOEN2 : OBUF port map (I=>xDIF_OEN(2), O=>dif_oen(2));
  oOEN3 : OBUF port map (I=>xDIF_OEN(3), O=>dif_oen(3));
  oOEN4 : OBUF port map (I=>xDIF_OEN(4), O=>dif_oen(4));
  oOEN5 : OBUF port map (I=>xDIF_OEN(5), O=>dif_oen(5));

  -------------------------------------------------------------
  -- FIFO output

  iWR : HE_WR_6F
    port map (
      DOF_FF        => xDOF_FF,
      DOF_AF        => xDOF_AF,
      DOF_WEN       => xDOF_WEN,
      DO            => xDO,
      --
      RST           => RESET,
      CLK           => FCLK_WR,
      OUTFIFO_WRITE => OUTFIFO_WRITE,
      OUTFIFO_D     => OUTFIFO_D,
      OUTFIFO_READY => OUTFIFO_READY );

  oDO0  : OBUF port map (I=>xDO(0),  O=>do(0));
  oDO1  : OBUF port map (I=>xDO(1),  O=>do(1));
  oDO2  : OBUF port map (I=>xDO(2),  O=>do(2));
  oDO3  : OBUF port map (I=>xDO(3),  O=>do(3));
  oDO4  : OBUF port map (I=>xDO(4),  O=>do(4));
  oDO5  : OBUF port map (I=>xDO(5),  O=>do(5));
  oDO6  : OBUF port map (I=>xDO(6),  O=>do(6));
  oDO7  : OBUF port map (I=>xDO(7),  O=>do(7));

  oDO8  : OBUF port map (I=>xDO(8),  O=>do(8));
  oDO9  : OBUF port map (I=>xDO(9),  O=>do(9));
  oDO10 : OBUF port map (I=>xDO(10), O=>do(10));
  oDO11 : OBUF port map (I=>xDO(11), O=>do(11));
  oDO12 : OBUF port map (I=>xDO(12), O=>do(12));
  oDO13 : OBUF port map (I=>xDO(13), O=>do(13));
  oDO14 : OBUF port map (I=>xDO(14), O=>do(14));
  oDO15 : OBUF port map (I=>xDO(15), O=>do(15));

  oDO16 : OBUF port map (I=>xDO(16), O=>do(16));
  oDO17 : OBUF port map (I=>xDO(17), O=>do(17));
  oDO18 : OBUF port map (I=>xDO(18), O=>do(18));
  oDO19 : OBUF port map (I=>xDO(19), O=>do(19));
  oDO20 : OBUF port map (I=>xDO(20), O=>do(20));
  oDO21 : OBUF port map (I=>xDO(21), O=>do(21));
  oDO22 : OBUF port map (I=>xDO(22), O=>do(22));
  oDO23 : OBUF port map (I=>xDO(23), O=>do(23));

  oDO24 : OBUF port map (I=>xDO(24), O=>do(24));
  oDO25 : OBUF port map (I=>xDO(25), O=>do(25));
  oDO26 : OBUF port map (I=>xDO(26), O=>do(26));
  oDO27 : OBUF port map (I=>xDO(27), O=>do(27));
  oDO28 : OBUF port map (I=>xDO(28), O=>do(28));
  oDO29 : OBUF port map (I=>xDO(29), O=>do(29));
  oDO30 : OBUF port map (I=>xDO(30), O=>do(30));
  oDO31 : OBUF port map (I=>xDO(31), O=>do(31));

  iFF0 : IBUF port map (I=>dof_ff(0), O=>xDOF_FF(0));
  iFF1 : IBUF port map (I=>dof_ff(1), O=>xDOF_FF(1));
  iFF2 : IBUF port map (I=>dof_ff(2), O=>xDOF_FF(2));
  iFF3 : IBUF port map (I=>dof_ff(3), O=>xDOF_FF(3));
  iFF4 : IBUF port map (I=>dof_ff(4), O=>xDOF_FF(4));
  iFF5 : IBUF port map (I=>dof_ff(5), O=>xDOF_FF(5));

  iAF0 : IBUF port map (I=>dof_af(0), O=>xDOF_AF(0));
  iAF1 : IBUF port map (I=>dof_af(1), O=>xDOF_AF(1));
  iAF2 : IBUF port map (I=>dof_af(2), O=>xDOF_AF(2));
  iAF3 : IBUF port map (I=>dof_af(3), O=>xDOF_AF(3));
  iAF4 : IBUF port map (I=>dof_af(4), O=>xDOF_AF(4));
  iAF5 : IBUF port map (I=>dof_af(5), O=>xDOF_AF(5));

  oWEN0 : OBUF port map (I=>xDOF_WEN(0), O=>dof_wen(0));
  oWEN1 : OBUF port map (I=>xDOF_WEN(1), O=>dof_wen(1));
  oWEN2 : OBUF port map (I=>xDOF_WEN(2), O=>dof_wen(2));
  oWEN3 : OBUF port map (I=>xDOF_WEN(3), O=>dof_wen(3));
  oWEN4 : OBUF port map (I=>xDOF_WEN(4), O=>dof_wen(4));
  oWEN5 : OBUF port map (I=>xDOF_WEN(5), O=>dof_wen(5));

  -------------------------------------------------------------
  -- FLASH

  FLASH_ADDR(24) <= '0';

  oFL_A0  : OBUF port map (I=>FLASH_ADDR(0),  O=>flash_a(0));
  oFL_A1  : OBUF port map (I=>FLASH_ADDR(1),  O=>flash_a(1));
  oFL_A2  : OBUF port map (I=>FLASH_ADDR(2),  O=>flash_a(2));
  oFL_A3  : OBUF port map (I=>FLASH_ADDR(3),  O=>flash_a(3));
  oFL_A4  : OBUF port map (I=>FLASH_ADDR(4),  O=>flash_a(4));
  oFL_A5  : OBUF port map (I=>FLASH_ADDR(5),  O=>flash_a(5));
  oFL_A6  : OBUF port map (I=>FLASH_ADDR(6),  O=>flash_a(6));
  oFL_A7  : OBUF port map (I=>FLASH_ADDR(7),  O=>flash_a(7));
  oFL_A8  : OBUF port map (I=>FLASH_ADDR(8),  O=>flash_a(8));
  oFL_A9  : OBUF port map (I=>FLASH_ADDR(9),  O=>flash_a(9));
  oFL_A10 : OBUF port map (I=>FLASH_ADDR(10), O=>flash_a(10));
  oFL_A11 : OBUF port map (I=>FLASH_ADDR(11), O=>flash_a(11));
  oFL_A12 : OBUF port map (I=>FLASH_ADDR(12), O=>flash_a(12));
  oFL_A13 : OBUF port map (I=>FLASH_ADDR(13), O=>flash_a(13));
  oFL_A14 : OBUF port map (I=>FLASH_ADDR(14), O=>flash_a(14));
  oFL_A15 : OBUF port map (I=>FLASH_ADDR(15), O=>flash_a(15));
  oFL_A16 : OBUF port map (I=>FLASH_ADDR(16), O=>flash_a(16));
  oFL_A17 : OBUF port map (I=>FLASH_ADDR(17), O=>flash_a(17));
  oFL_A18 : OBUF port map (I=>FLASH_ADDR(18), O=>flash_a(18));
  oFL_A19 : OBUF port map (I=>FLASH_ADDR(19), O=>flash_a(19));
  oFL_A20 : OBUF port map (I=>FLASH_ADDR(20), O=>flash_a(20));
  oFL_A21 : OBUF port map (I=>FLASH_ADDR(21), O=>flash_a(21));
  oFL_A22 : OBUF port map (I=>FLASH_ADDR(22), O=>flash_a(22));
  oFL_A23 : OBUF port map (I=>FLASH_ADDR(23), O=>flash_a(23));
  oFL_A24 : OBUF port map (I=>FLASH_ADDR(24), O=>flash_a(24));

  iFL_D0 : iobuf port map (T=>FLASH_EN(0), I=>FLASH_OUT(0), O=>FLASH_IN(0), IO=>flash_d(0));
  iFL_D1 : iobuf port map (T=>FLASH_EN(1), I=>FLASH_OUT(1), O=>FLASH_IN(1), IO=>flash_d(1));
  iFL_D2 : iobuf port map (T=>FLASH_EN(2), I=>FLASH_OUT(2), O=>FLASH_IN(2), IO=>flash_d(2));
  iFL_D3 : iobuf port map (T=>FLASH_EN(3), I=>FLASH_OUT(3), O=>FLASH_IN(3), IO=>flash_d(3));
  iFL_D4 : iobuf port map (T=>FLASH_EN(4), I=>FLASH_OUT(4), O=>FLASH_IN(4), IO=>flash_d(4));
  iFL_D5 : iobuf port map (T=>FLASH_EN(5), I=>FLASH_OUT(5), O=>FLASH_IN(5), IO=>flash_d(5));
  iFL_D6 : iobuf port map (T=>FLASH_EN(6), I=>FLASH_OUT(6), O=>FLASH_IN(6), IO=>flash_d(6));
  iFL_D7 : iobuf port map (T=>FLASH_EN(7), I=>FLASH_OUT(7), O=>FLASH_IN(7), IO=>flash_d(7));

  oFL_CE : OBUF port map (I=>FLASH_CE, O=>flash_cen);
  oFL_WE : OBUF port map (I=>FLASH_WE, O=>flash_wen);
  oFL_OE : OBUF port map (I=>FLASH_OE, O=>flash_oen);

  oFL_RP : OBUF port map (I=>FLASH_RP,   O=>flash_rpn);
  oFL_VP : OBUF port map (I=>FLASH_VPEN, O=>flash_vp);

  iFL_ST : IBUF port map (I=>flash_st, O=>FLASH_STS);

  -------------------------------------------------------------
  -- DDR SDRAM

  iDDR : HE_DDR
    port map (
      ddr_a_clk         => ddr_a_clk,
      ddr_a_clkb        => ddr_a_clkb,
      ddr_a_address     => ddr_a_address,
      ddr_a_BA          => ddr_a_BA,
      ddr_a_dq_in       => ddr_a_dq_in,
      ddr_a_dq_out      => ddr_a_dq_out,
      ddr_a_dq_en       => ddr_a_dq_en,
      ddr_a_dqs_in      => ddr_a_dqs_in,
      ddr_a_dqs_out     => ddr_a_dqs_out,
      ddr_a_dqs_en      => ddr_a_dqs_en,
      ddr_a_dm_out      => ddr_a_dm_out,
      ddr_a_dm_en       => ddr_a_dm_en,
      ddr_a_cke         => ddr_a_cke,
      ddr_a_csb         => ddr_a_csb,
      ddr_a_rasb        => ddr_a_rasb,
      ddr_a_casb        => ddr_a_casb,
      ddr_a_web         => ddr_a_web,
      --
      ddr_b_clk         => ddr_b_clk,
      ddr_b_clkb        => ddr_b_clkb,
      ddr_b_address     => ddr_b_address,
      ddr_b_BA          => ddr_b_BA,
      ddr_b_dq_in       => ddr_b_dq_in,
      ddr_b_dq_out      => ddr_b_dq_out,
      ddr_b_dq_en       => ddr_b_dq_en,
      ddr_b_dqs_in      => ddr_b_dqs_in,
      ddr_b_dqs_out     => ddr_b_dqs_out,
      ddr_b_dqs_en      => ddr_b_dqs_en,
      ddr_b_dm_out      => ddr_b_dm_out,
      ddr_b_dm_en       => ddr_b_dm_en,
      ddr_b_cke         => ddr_b_cke,
      ddr_b_csb         => ddr_b_csb,
      ddr_b_rasb        => ddr_b_rasb,
      ddr_b_casb        => ddr_b_casb,
      ddr_b_web         => ddr_b_web,
      --
      user_input_clk    => user_input_clk,
      user_rst          => RESET,
      --
      DDR_A_USER_CLK    => DDR_A_USER_CLK,
      --
      DDR_A_USER_RST    => DDR_A_USER_RST,
      --
      DDR_A_DATA_COUNT  => DDR_A_DATA_COUNT,
      --
      DDR_A_WR_DATA     => DDR_A_WR_DATA,
      DDR_A_WR_ADDR     => DDR_A_WR_ADDR,
      DDR_A_WR_ADDR_WEN => DDR_A_WR_ADDR_WEN,
      DDR_A_WR_ADDR_FF  => DDR_A_WR_ADDR_FF,
      DDR_A_WR_ADDR_AF  => DDR_A_WR_ADDR_AF,
      DDR_A_WR_RISE_WEN => DDR_A_WR_RISE_WEN,
      DDR_A_WR_RISE_FF  => DDR_A_WR_RISE_FF,
      DDR_A_WR_RISE_AF  => DDR_A_WR_RISE_AF,
      DDR_A_WR_FALL_WEN => DDR_A_WR_FALL_WEN,
      DDR_A_WR_FALL_FF  => DDR_A_WR_FALL_FF,
      DDR_A_WR_FALL_AF  => DDR_A_WR_FALL_AF,
      DDR_A_WR_DM_WEN   => DDR_A_WR_DM_WEN,
      DDR_A_WR_DM_FF    => DDR_A_WR_DM_FF,
      DDR_A_WR_DM_AF    => DDR_A_WR_DM_AF,
      --
      DDR_A_RD_DATA     => DDR_A_RD_DATA,
      DDR_A_RD_ADDR     => DDR_A_RD_ADDR,
      DDR_A_RD_ADDR_WEN => DDR_A_RD_ADDR_WEN,
      DDR_A_RD_ADDR_FF  => DDR_A_RD_ADDR_FF,
      DDR_A_RD_ADDR_AF  => DDR_A_RD_ADDR_AF,
      DDR_A_RD_RISE_REN => DDR_A_RD_RISE_REN,
      DDR_A_RD_RISE_EF  => DDR_A_RD_RISE_EF,
      DDR_A_RD_RISE_AE  => DDR_A_RD_RISE_AE,
      DDR_A_RD_FALL_REN => DDR_A_RD_FALL_REN,
      DDR_A_RD_FALL_EF  => DDR_A_RD_FALL_EF,
      DDR_A_RD_FALL_AE  => DDR_A_RD_FALL_AE,
      --
      DDR_B_USER_CLK    => DDR_B_USER_CLK,
      --
      DDR_B_USER_RST    => DDR_B_USER_RST,
      --
      DDR_B_DATA_COUNT  => DDR_B_DATA_COUNT,
      --
      DDR_B_WR_DATA     => DDR_B_WR_DATA,
      DDR_B_WR_ADDR     => DDR_B_WR_ADDR,
      DDR_B_WR_ADDR_WEN => DDR_B_WR_ADDR_WEN,
      DDR_B_WR_ADDR_FF  => DDR_B_WR_ADDR_FF,
      DDR_B_WR_ADDR_AF  => DDR_B_WR_ADDR_AF,
      DDR_B_WR_RISE_WEN => DDR_B_WR_RISE_WEN,
      DDR_B_WR_RISE_FF  => DDR_B_WR_RISE_FF,
      DDR_B_WR_RISE_AF  => DDR_B_WR_RISE_AF,
      DDR_B_WR_FALL_WEN => DDR_B_WR_FALL_WEN,
      DDR_B_WR_FALL_FF  => DDR_B_WR_FALL_FF,
      DDR_B_WR_FALL_AF  => DDR_B_WR_FALL_AF,
      DDR_B_WR_DM_WEN   => DDR_B_WR_DM_WEN,
      DDR_B_WR_DM_FF    => DDR_B_WR_DM_FF,
      DDR_B_WR_DM_AF    => DDR_B_WR_DM_AF,
      --
      DDR_B_RD_DATA     => DDR_B_RD_DATA,
      DDR_B_RD_ADDR     => DDR_B_RD_ADDR,
      DDR_B_RD_ADDR_WEN => DDR_B_RD_ADDR_WEN,
      DDR_B_RD_ADDR_FF  => DDR_B_RD_ADDR_FF,
      DDR_B_RD_ADDR_AF  => DDR_B_RD_ADDR_AF,
      DDR_B_RD_RISE_REN => DDR_B_RD_RISE_REN,
      DDR_B_RD_RISE_EF  => DDR_B_RD_RISE_EF,
      DDR_B_RD_RISE_AE  => DDR_B_RD_RISE_AE,
      DDR_B_RD_FALL_REN => DDR_B_RD_FALL_REN,
      DDR_B_RD_FALL_EF  => DDR_B_RD_FALL_EF,
      DDR_B_RD_FALL_AE  => DDR_B_RD_FALL_AE,
      --
      PPC_TO_DDR_IF_A   => PPC_TO_DDR_IF_A,
      DDR_TO_PPC_IF_A   => DDR_TO_PPC_IF_A,
      --
      PPC_TO_DDR_IF_B   => PPC_TO_DDR_IF_B,
      DDR_TO_PPC_IF_B   => DDR_TO_PPC_IF_B );

  iSCLKI : IBUFG port map (I=>sdram_clkin, O=>user_input_clk);

  oSCLK_A0 : OBUF port map (I=>ddr_a_clk,  O=>sdram_a_clk_p);
  oSCLK_A1 : OBUF port map (I=>ddr_a_clkb, O=>sdram_a_clk_n);

  oSCLK_B0 : OBUF port map (I=>ddr_b_clk,  O=>sdram_b_clk_p);
  oSCLK_B1 : OBUF port map (I=>ddr_b_clkb, O=>sdram_b_clk_n);

  oSD_A_AD0  : OBUF port map (I=>ddr_a_address(0),  O=>sdram_a_a(0));
  oSD_A_AD1  : OBUF port map (I=>ddr_a_address(1),  O=>sdram_a_a(1));
  oSD_A_AD2  : OBUF port map (I=>ddr_a_address(2),  O=>sdram_a_a(2));
  oSD_A_AD3  : OBUF port map (I=>ddr_a_address(3),  O=>sdram_a_a(3));
  oSD_A_AD4  : OBUF port map (I=>ddr_a_address(4),  O=>sdram_a_a(4));
  oSD_A_AD5  : OBUF port map (I=>ddr_a_address(5),  O=>sdram_a_a(5));
  oSD_A_AD6  : OBUF port map (I=>ddr_a_address(6),  O=>sdram_a_a(6));
  oSD_A_AD7  : OBUF port map (I=>ddr_a_address(7),  O=>sdram_a_a(7));
  oSD_A_AD8  : OBUF port map (I=>ddr_a_address(8),  O=>sdram_a_a(8));
  oSD_A_AD9  : OBUF port map (I=>ddr_a_address(9),  O=>sdram_a_a(9));
  oSD_A_AD10 : OBUF port map (I=>ddr_a_address(10), O=>sdram_a_a(10));
  oSD_A_AD11 : OBUF port map (I=>ddr_a_address(11), O=>sdram_a_a(11));
  oSD_A_AD12 : OBUF port map (I=>ddr_a_address(12), O=>sdram_a_a(12));

  oSD_B_AD0  : OBUF port map (I=>ddr_b_address(0),  O=>sdram_b_a(0));
  oSD_B_AD1  : OBUF port map (I=>ddr_b_address(1),  O=>sdram_b_a(1));
  oSD_B_AD2  : OBUF port map (I=>ddr_b_address(2),  O=>sdram_b_a(2));
  oSD_B_AD3  : OBUF port map (I=>ddr_b_address(3),  O=>sdram_b_a(3));
  oSD_B_AD4  : OBUF port map (I=>ddr_b_address(4),  O=>sdram_b_a(4));
  oSD_B_AD5  : OBUF port map (I=>ddr_b_address(5),  O=>sdram_b_a(5));
  oSD_B_AD6  : OBUF port map (I=>ddr_b_address(6),  O=>sdram_b_a(6));
  oSD_B_AD7  : OBUF port map (I=>ddr_b_address(7),  O=>sdram_b_a(7));
  oSD_B_AD8  : OBUF port map (I=>ddr_b_address(8),  O=>sdram_b_a(8));
  oSD_B_AD9  : OBUF port map (I=>ddr_b_address(9),  O=>sdram_b_a(9));
  oSD_B_AD10 : OBUF port map (I=>ddr_b_address(10), O=>sdram_b_a(10));
  oSD_B_AD11 : OBUF port map (I=>ddr_b_address(11), O=>sdram_b_a(11));
  oSD_B_AD12 : OBUF port map (I=>ddr_b_address(12), O=>sdram_b_a(12));

  oSD_A_BA0 : OBUF port map (I=>ddr_a_BA(0), O=>sdram_a_ba(0));
  oSD_A_BA1 : OBUF port map (I=>ddr_a_BA(1), O=>sdram_a_ba(1));

  oSD_B_BA0 : OBUF port map (I=>ddr_b_BA(0), O=>sdram_b_ba(0));
  oSD_B_BA1 : OBUF port map (I=>ddr_b_BA(1), O=>sdram_b_ba(1));

  iSD_A_DXI : IBUF port map (I=>sdram_a_dxi, O=>ddr_a_dx);
  oSD_A_DXO : OBUF port map (I=>ddr_a_dx, O=>sdram_a_dxo);

  iSD_B_DXI : IBUF port map (I=>sdram_b_dxi, O=>ddr_b_dx);
  oSD_B_DXO : OBUF port map (I=>ddr_b_dx, O=>sdram_b_dxo);

  iSD_A_D0  : IOBUF port map (T=>ddr_a_dq_en(0),  I=>ddr_a_dq_out(0),  O=>ddr_a_dq_in(0),  IO=>sdram_a_dq(0));
  iSD_A_D1  : IOBUF port map (T=>ddr_a_dq_en(1),  I=>ddr_a_dq_out(1),  O=>ddr_a_dq_in(1),  IO=>sdram_a_dq(1));
  iSD_A_D2  : IOBUF port map (T=>ddr_a_dq_en(2),  I=>ddr_a_dq_out(2),  O=>ddr_a_dq_in(2),  IO=>sdram_a_dq(2));
  iSD_A_D3  : IOBUF port map (T=>ddr_a_dq_en(3),  I=>ddr_a_dq_out(3),  O=>ddr_a_dq_in(3),  IO=>sdram_a_dq(3));
  iSD_A_D4  : IOBUF port map (T=>ddr_a_dq_en(4),  I=>ddr_a_dq_out(4),  O=>ddr_a_dq_in(4),  IO=>sdram_a_dq(4));
  iSD_A_D5  : IOBUF port map (T=>ddr_a_dq_en(5),  I=>ddr_a_dq_out(5),  O=>ddr_a_dq_in(5),  IO=>sdram_a_dq(5));
  iSD_A_D6  : IOBUF port map (T=>ddr_a_dq_en(6),  I=>ddr_a_dq_out(6),  O=>ddr_a_dq_in(6),  IO=>sdram_a_dq(6));
  iSD_A_D7  : IOBUF port map (T=>ddr_a_dq_en(7),  I=>ddr_a_dq_out(7),  O=>ddr_a_dq_in(7),  IO=>sdram_a_dq(7));

  iSD_A_D8  : IOBUF port map (T=>ddr_a_dq_en(8),  I=>ddr_a_dq_out(8),  O=>ddr_a_dq_in(8),  IO=>sdram_a_dq(8));
  iSD_A_D9  : IOBUF port map (T=>ddr_a_dq_en(9),  I=>ddr_a_dq_out(9),  O=>ddr_a_dq_in(9),  IO=>sdram_a_dq(9));
  iSD_A_D10 : IOBUF port map (T=>ddr_a_dq_en(10), I=>ddr_a_dq_out(10), O=>ddr_a_dq_in(10), IO=>sdram_a_dq(10));
  iSD_A_D11 : IOBUF port map (T=>ddr_a_dq_en(11), I=>ddr_a_dq_out(11), O=>ddr_a_dq_in(11), IO=>sdram_a_dq(11));
  iSD_A_D12 : IOBUF port map (T=>ddr_a_dq_en(12), I=>ddr_a_dq_out(12), O=>ddr_a_dq_in(12), IO=>sdram_a_dq(12));
  iSD_A_D13 : IOBUF port map (T=>ddr_a_dq_en(13), I=>ddr_a_dq_out(13), O=>ddr_a_dq_in(13), IO=>sdram_a_dq(13));
  iSD_A_D14 : IOBUF port map (T=>ddr_a_dq_en(14), I=>ddr_a_dq_out(14), O=>ddr_a_dq_in(14), IO=>sdram_a_dq(14));
  iSD_A_D15 : IOBUF port map (T=>ddr_a_dq_en(15), I=>ddr_a_dq_out(15), O=>ddr_a_dq_in(15), IO=>sdram_a_dq(15));

  iSD_A_D16 : IOBUF port map (T=>ddr_a_dq_en(16), I=>ddr_a_dq_out(16), O=>ddr_a_dq_in(16), IO=>sdram_a_dq(16));
  iSD_A_D17 : IOBUF port map (T=>ddr_a_dq_en(17), I=>ddr_a_dq_out(17), O=>ddr_a_dq_in(17), IO=>sdram_a_dq(17));
  iSD_A_D18 : IOBUF port map (T=>ddr_a_dq_en(18), I=>ddr_a_dq_out(18), O=>ddr_a_dq_in(18), IO=>sdram_a_dq(18));
  iSD_A_D19 : IOBUF port map (T=>ddr_a_dq_en(19), I=>ddr_a_dq_out(19), O=>ddr_a_dq_in(19), IO=>sdram_a_dq(19));
  iSD_A_D20 : IOBUF port map (T=>ddr_a_dq_en(20), I=>ddr_a_dq_out(20), O=>ddr_a_dq_in(20), IO=>sdram_a_dq(20));
  iSD_A_D21 : IOBUF port map (T=>ddr_a_dq_en(21), I=>ddr_a_dq_out(21), O=>ddr_a_dq_in(21), IO=>sdram_a_dq(21));
  iSD_A_D22 : IOBUF port map (T=>ddr_a_dq_en(22), I=>ddr_a_dq_out(22), O=>ddr_a_dq_in(22), IO=>sdram_a_dq(22));
  iSD_A_D23 : IOBUF port map (T=>ddr_a_dq_en(23), I=>ddr_a_dq_out(23), O=>ddr_a_dq_in(23), IO=>sdram_a_dq(23));

  iSD_A_D24 : IOBUF port map (T=>ddr_a_dq_en(24), I=>ddr_a_dq_out(24), O=>ddr_a_dq_in(24), IO=>sdram_a_dq(24));
  iSD_A_D25 : IOBUF port map (T=>ddr_a_dq_en(25), I=>ddr_a_dq_out(25), O=>ddr_a_dq_in(25), IO=>sdram_a_dq(25));
  iSD_A_D26 : IOBUF port map (T=>ddr_a_dq_en(26), I=>ddr_a_dq_out(26), O=>ddr_a_dq_in(26), IO=>sdram_a_dq(26));
  iSD_A_D27 : IOBUF port map (T=>ddr_a_dq_en(27), I=>ddr_a_dq_out(27), O=>ddr_a_dq_in(27), IO=>sdram_a_dq(27));
  iSD_A_D28 : IOBUF port map (T=>ddr_a_dq_en(28), I=>ddr_a_dq_out(28), O=>ddr_a_dq_in(28), IO=>sdram_a_dq(28));
  iSD_A_D29 : IOBUF port map (T=>ddr_a_dq_en(29), I=>ddr_a_dq_out(29), O=>ddr_a_dq_in(29), IO=>sdram_a_dq(29));
  iSD_A_D30 : IOBUF port map (T=>ddr_a_dq_en(30), I=>ddr_a_dq_out(30), O=>ddr_a_dq_in(30), IO=>sdram_a_dq(30));
  iSD_A_D31 : IOBUF port map (T=>ddr_a_dq_en(31), I=>ddr_a_dq_out(31), O=>ddr_a_dq_in(31), IO=>sdram_a_dq(31));

  iSD_B_D0  : IOBUF port map (T=>ddr_b_dq_en(0),  I=>ddr_b_dq_out(0),  O=>ddr_b_dq_in(0),  IO=>sdram_b_dq(0));
  iSD_B_D1  : IOBUF port map (T=>ddr_b_dq_en(1),  I=>ddr_b_dq_out(1),  O=>ddr_b_dq_in(1),  IO=>sdram_b_dq(1));
  iSD_B_D2  : IOBUF port map (T=>ddr_b_dq_en(2),  I=>ddr_b_dq_out(2),  O=>ddr_b_dq_in(2),  IO=>sdram_b_dq(2));
  iSD_B_D3  : IOBUF port map (T=>ddr_b_dq_en(3),  I=>ddr_b_dq_out(3),  O=>ddr_b_dq_in(3),  IO=>sdram_b_dq(3));
  iSD_B_D4  : IOBUF port map (T=>ddr_b_dq_en(4),  I=>ddr_b_dq_out(4),  O=>ddr_b_dq_in(4),  IO=>sdram_b_dq(4));
  iSD_B_D5  : IOBUF port map (T=>ddr_b_dq_en(5),  I=>ddr_b_dq_out(5),  O=>ddr_b_dq_in(5),  IO=>sdram_b_dq(5));
  iSD_B_D6  : IOBUF port map (T=>ddr_b_dq_en(6),  I=>ddr_b_dq_out(6),  O=>ddr_b_dq_in(6),  IO=>sdram_b_dq(6));
  iSD_B_D7  : IOBUF port map (T=>ddr_b_dq_en(7),  I=>ddr_b_dq_out(7),  O=>ddr_b_dq_in(7),  IO=>sdram_b_dq(7));

  iSD_B_D8  : IOBUF port map (T=>ddr_b_dq_en(8),  I=>ddr_b_dq_out(8),  O=>ddr_b_dq_in(8),  IO=>sdram_b_dq(8));
  iSD_B_D9  : IOBUF port map (T=>ddr_b_dq_en(9),  I=>ddr_b_dq_out(9),  O=>ddr_b_dq_in(9),  IO=>sdram_b_dq(9));
  iSD_B_D10 : IOBUF port map (T=>ddr_b_dq_en(10), I=>ddr_b_dq_out(10), O=>ddr_b_dq_in(10), IO=>sdram_b_dq(10));
  iSD_B_D11 : IOBUF port map (T=>ddr_b_dq_en(11), I=>ddr_b_dq_out(11), O=>ddr_b_dq_in(11), IO=>sdram_b_dq(11));
  iSD_B_D12 : IOBUF port map (T=>ddr_b_dq_en(12), I=>ddr_b_dq_out(12), O=>ddr_b_dq_in(12), IO=>sdram_b_dq(12));
  iSD_B_D13 : IOBUF port map (T=>ddr_b_dq_en(13), I=>ddr_b_dq_out(13), O=>ddr_b_dq_in(13), IO=>sdram_b_dq(13));
  iSD_B_D14 : IOBUF port map (T=>ddr_b_dq_en(14), I=>ddr_b_dq_out(14), O=>ddr_b_dq_in(14), IO=>sdram_b_dq(14));
  iSD_B_D15 : IOBUF port map (T=>ddr_b_dq_en(15), I=>ddr_b_dq_out(15), O=>ddr_b_dq_in(15), IO=>sdram_b_dq(15));

  iSD_B_D16 : IOBUF port map (T=>ddr_b_dq_en(16), I=>ddr_b_dq_out(16), O=>ddr_b_dq_in(16), IO=>sdram_b_dq(16));
  iSD_B_D17 : IOBUF port map (T=>ddr_b_dq_en(17), I=>ddr_b_dq_out(17), O=>ddr_b_dq_in(17), IO=>sdram_b_dq(17));
  iSD_B_D18 : IOBUF port map (T=>ddr_b_dq_en(18), I=>ddr_b_dq_out(18), O=>ddr_b_dq_in(18), IO=>sdram_b_dq(18));
  iSD_B_D19 : IOBUF port map (T=>ddr_b_dq_en(19), I=>ddr_b_dq_out(19), O=>ddr_b_dq_in(19), IO=>sdram_b_dq(19));
  iSD_B_D20 : IOBUF port map (T=>ddr_b_dq_en(20), I=>ddr_b_dq_out(20), O=>ddr_b_dq_in(20), IO=>sdram_b_dq(20));
  iSD_B_D21 : IOBUF port map (T=>ddr_b_dq_en(21), I=>ddr_b_dq_out(21), O=>ddr_b_dq_in(21), IO=>sdram_b_dq(21));
  iSD_B_D22 : IOBUF port map (T=>ddr_b_dq_en(22), I=>ddr_b_dq_out(22), O=>ddr_b_dq_in(22), IO=>sdram_b_dq(22));
  iSD_B_D23 : IOBUF port map (T=>ddr_b_dq_en(23), I=>ddr_b_dq_out(23), O=>ddr_b_dq_in(23), IO=>sdram_b_dq(23));

  iSD_B_D24 : IOBUF port map (T=>ddr_b_dq_en(24), I=>ddr_b_dq_out(24), O=>ddr_b_dq_in(24), IO=>sdram_b_dq(24));
  iSD_B_D25 : IOBUF port map (T=>ddr_b_dq_en(25), I=>ddr_b_dq_out(25), O=>ddr_b_dq_in(25), IO=>sdram_b_dq(25));
  iSD_B_D26 : IOBUF port map (T=>ddr_b_dq_en(26), I=>ddr_b_dq_out(26), O=>ddr_b_dq_in(26), IO=>sdram_b_dq(26));
  iSD_B_D27 : IOBUF port map (T=>ddr_b_dq_en(27), I=>ddr_b_dq_out(27), O=>ddr_b_dq_in(27), IO=>sdram_b_dq(27));
  iSD_B_D28 : IOBUF port map (T=>ddr_b_dq_en(28), I=>ddr_b_dq_out(28), O=>ddr_b_dq_in(28), IO=>sdram_b_dq(28));
  iSD_B_D29 : IOBUF port map (T=>ddr_b_dq_en(29), I=>ddr_b_dq_out(29), O=>ddr_b_dq_in(29), IO=>sdram_b_dq(29));
  iSD_B_D30 : IOBUF port map (T=>ddr_b_dq_en(30), I=>ddr_b_dq_out(30), O=>ddr_b_dq_in(30), IO=>sdram_b_dq(30));
  iSD_B_D31 : IOBUF port map (T=>ddr_b_dq_en(31), I=>ddr_b_dq_out(31), O=>ddr_b_dq_in(31), IO=>sdram_b_dq(31));

  iSD_A_DQS0 : IOBUF port map (T=>ddr_a_dqs_en(0), I=>ddr_a_dqs_out(0), O=>ddr_a_dqs_in(0), IO=>sdram_a_dqs(0));
  iSD_A_DQS1 : IOBUF port map (T=>ddr_a_dqs_en(1), I=>ddr_a_dqs_out(1), O=>ddr_a_dqs_in(1), IO=>sdram_a_dqs(1));
  iSD_A_DQS2 : IOBUF port map (T=>ddr_a_dqs_en(2), I=>ddr_a_dqs_out(2), O=>ddr_a_dqs_in(2), IO=>sdram_a_dqs(2));
  iSD_A_DQS3 : IOBUF port map (T=>ddr_a_dqs_en(3), I=>ddr_a_dqs_out(3), O=>ddr_a_dqs_in(3), IO=>sdram_a_dqs(3));

  iSD_B_DQS0 : IOBUF port map (T=>ddr_b_dqs_en(0), I=>ddr_b_dqs_out(0), O=>ddr_b_dqs_in(0), IO=>sdram_b_dqs(0));
  iSD_B_DQS1 : IOBUF port map (T=>ddr_b_dqs_en(1), I=>ddr_b_dqs_out(1), O=>ddr_b_dqs_in(1), IO=>sdram_b_dqs(1));
  iSD_B_DQS2 : IOBUF port map (T=>ddr_b_dqs_en(2), I=>ddr_b_dqs_out(2), O=>ddr_b_dqs_in(2), IO=>sdram_b_dqs(2));
  iSD_B_DQS3 : IOBUF port map (T=>ddr_b_dqs_en(3), I=>ddr_b_dqs_out(3), O=>ddr_b_dqs_in(3), IO=>sdram_b_dqs(3));

  oSD_A_DM0 : OBUFT port map (I=>ddr_a_dm_out(0), T=>ddr_a_dm_en(0), O=>sdram_a_dm(0));
  oSD_A_DM1 : OBUFT port map (I=>ddr_a_dm_out(1), T=>ddr_a_dm_en(1), O=>sdram_a_dm(1));
  oSD_A_DM2 : OBUFT port map (I=>ddr_a_dm_out(2), T=>ddr_a_dm_en(2), O=>sdram_a_dm(2));
  oSD_A_DM3 : OBUFT port map (I=>ddr_a_dm_out(3), T=>ddr_a_dm_en(3), O=>sdram_a_dm(3));

  oSD_B_DM0 : OBUFT port map (I=>ddr_b_dm_out(0), T=>ddr_b_dm_en(0), O=>sdram_b_dm(0));
  oSD_B_DM1 : OBUFT port map (I=>ddr_b_dm_out(1), T=>ddr_b_dm_en(1), O=>sdram_b_dm(1));
  oSD_B_DM2 : OBUFT port map (I=>ddr_b_dm_out(2), T=>ddr_b_dm_en(2), O=>sdram_b_dm(2));
  oSD_B_DM3 : OBUFT port map (I=>ddr_b_dm_out(3), T=>ddr_b_dm_en(3), O=>sdram_b_dm(3));

  oSD_A_CKE : OBUF port map (I=>ddr_a_cke,  O=>sdram_a_cke);
  oSD_A_CS  : OBUF port map (I=>ddr_a_csb,  O=>sdram_a_cs);

  oSD_A_RAS : OBUF port map (I=>ddr_a_rasb, O=>sdram_a_ras);
  oSD_A_CAS : OBUF port map (I=>ddr_a_casb, O=>sdram_a_cas);
  oSD_A_WE  : OBUF port map (I=>ddr_a_web,  O=>sdram_a_we);

  oSD_B_CKE : OBUF port map (I=>ddr_b_cke,  O=>sdram_b_cke);
  oSD_B_CS  : OBUF port map (I=>ddr_b_csb,  O=>sdram_b_cs);

  oSD_B_RAS : OBUF port map (I=>ddr_b_rasb, O=>sdram_b_ras);
  oSD_B_CAS : OBUF port map (I=>ddr_b_casb, O=>sdram_b_cas);
  oSD_B_WE  : OBUF port map (I=>ddr_b_web,  O=>sdram_b_we);

  -------------------------------------------------------------
  -- Serial Bus Interface

  iUSR : HE_USER
    port map (
      VPD_IN     => VPD_IN,
      VPD_OUT    => VPD_OUT,
      VPD_EN     => VPD_EN,
      VINIT_IN   => VINIT_IN,
      VINIT_OUT  => VINIT_OUT,
      VINIT_EN   => VINIT_EN,
      VCS_IN     => VCS_IN,
      VCS_OUT    => VCS_OUT,
      VCS_EN     => VCS_EN,
      VWRITE_IN  => VWRITE_IN,
      VWRITE_OUT => VWRITE_OUT,
      VWRITE_EN  => VWRITE_EN,
      VBUSY_IN   => VBUSY_IN,
      VBUSY_OUT  => VBUSY_OUT,
      VBUSY_EN   => VBUSY_EN,
      --
      RST        => RESET,
      CLK        => MSG_CLK,
      READY      => MSG_READY,
      SEND_MSG   => MSG_SEND_MSG,
      CE         => MSG_CE,
      DOUT       => MSG_DOUT,
      CID        => xCID,
      MID        => xMID(2 downto 0),
      SEND_ID    => MSG_SEND_ID,
      LAST_BYTE  => MSG_LAST_BYTE,
      DIN        => MSG_DIN,
      ADDR       => MSG_ADDR,
      AEN        => MSG_AEN,
      WEN        => MSG_WEN,
      REN        => MSG_REN,
      DONE       => MSG_DONE,
      COUNT      => MSG_COUNT,
      CLEAR      => MSG_CLEAR );

  iVPD0 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(0), O=>VPD_IN(0), IO=>vpd(0));
  iVPD1 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(1), O=>VPD_IN(1), IO=>vpd(1));
  iVPD2 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(2), O=>VPD_IN(2), IO=>vpd(2));
  iVPD3 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(3), O=>VPD_IN(3), IO=>vpd(3));
  iVPD4 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(4), O=>VPD_IN(4), IO=>vpd(4));
  iVPD5 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(5), O=>VPD_IN(5), IO=>vpd(5));
  iVPD6 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(6), O=>VPD_IN(6), IO=>vpd(6));
  iVPD7 : iobuf port map (T=>VPD_EN, I=>VPD_OUT(7), O=>VPD_IN(7), IO=>vpd(7));

  iVINI : iobuf port map (T=>VINIT_EN,  I=>VINIT_OUT,  O=>VINIT_IN,  IO=>vinit);
  iVCS  : iobuf port map (T=>VCS_EN,    I=>VCS_OUT,    O=>VCS_IN,    IO=>vcs);
  iVWR  : iobuf port map (T=>VWRITE_EN, I=>VWRITE_OUT, O=>VWRITE_IN, IO=>vwrite);
  iVBSY : iobuf port map (T=>VBUSY_EN,  I=>VBUSY_OUT,  O=>VBUSY_IN,  IO=>vbusy);

  -------------------------------------------------------------
  -- Camera Link connections

  -- Camera Link Clock
  lvds_ibufg  : IBUFGDS port map (I=>dio_a(7), IB=>dio_a(6), O=>CAM_CLK);

  -- Camera Link Data
  lvds_ibuf0  : IBUFDS port map (I=>dio_a(1), IB=>dio_a(0), O=>CAM_DATA(0));
  lvds_ibuf1  : IBUFDS port map (I=>dio_a(3), IB=>dio_a(2), O=>CAM_DATA(1));
  lvds_ibuf2  : IBUFDS port map (I=>dio_a(5), IB=>dio_a(4), O=>CAM_DATA(2));
  lvds_ibuf3  : IBUFDS port map (I=>dio_a(9), IB=>dio_a(8), O=>CAM_DATA(3));

  -- Camera Link Control
  lvds_obuf0  : OBUFDS port map (I=>CAM_CTRL(0), O=>dio_b(5),  OB=>dio_b(4));
  lvds_obuf1  : OBUFDS port map (I=>CAM_CTRL(1), O=>dio_b(7),  OB=>dio_b(6));
  lvds_obuf2  : OBUFDS port map (I=>CAM_CTRL(2), O=>dio_b(9),  OB=>dio_b(8));
  lvds_obuf3  : OBUFDS port map (I=>CAM_CTRL(3), O=>dio_b(11), OB=>dio_b(10));

  -- Camera Link RS232 connection from HERON-FPGAx to camera
  lvds_obuf4  : OBUFDS port map (I=>CAM_SER_TC, O=>dio_b(1), OB=>dio_b(0));

  -- Camera Link RS232 connection from camera to HERON-FPGAx
  lvds_ibuf4  : IBUFDS port map (I=>dio_b(3), IB=>dio_b(2), O=>CAM_SER_TFG);

  -------------------------------------------------------------
  -- Connector Digital IO

  -- NOTE : Signals 0 to 9 on Connector A are used for Camera Link
  iDIOA10 : iobuf port map (T=>CONN_A_EN(10), I=>CONN_A_OUT(10), O=>CONN_A_IN(10), IO=>dio_a(10));
  iDIOA11 : iobuf port map (T=>CONN_A_EN(11), I=>CONN_A_OUT(11), O=>CONN_A_IN(11), IO=>dio_a(11));
  iDIOA12 : iobuf port map (T=>CONN_A_EN(12), I=>CONN_A_OUT(12), O=>CONN_A_IN(12), IO=>dio_a(12));
  iDIOA13 : iobuf port map (T=>CONN_A_EN(13), I=>CONN_A_OUT(13), O=>CONN_A_IN(13), IO=>dio_a(13));
  iDIOA14 : iobuf port map (T=>CONN_A_EN(14), I=>CONN_A_OUT(14), O=>CONN_A_IN(14), IO=>dio_a(14));

  -- NOTE : Signals 0 to 11 on Connector B are used for Camera Link
  iDIOB12 : iobuf port map (T=>CONN_B_EN(12), I=>CONN_B_OUT(12), O=>CONN_B_IN(12), IO=>dio_b(12));
  iDIOB13 : iobuf port map (T=>CONN_B_EN(13), I=>CONN_B_OUT(13), O=>CONN_B_IN(13), IO=>dio_b(13));
  iDIOB14 : iobuf port map (T=>CONN_B_EN(14), I=>CONN_B_OUT(14), O=>CONN_B_IN(14), IO=>dio_b(14));

  -------------------------------------------------------------
  -- USER_AP

  iuser_ap : USER_AP
    port map (
      RESET             => RESET,
      CONFIG            => xCONFIG,
      --
      OSC0              => OSC0,
      OSC3              => OSC3,
      --
      UDPRES            => xUDPRES,
      LED               => xLED,
      --
      ADDR_FLAGSEL      => xADDR_FLAGSEL,
      BOOTEN            => xBOOTEN,
      --
      CID               => xCID,
      MID               => xMID,
      UMI_IN            => UMI_IN,
      UMI_OUT           => UMI_OUT,
      UMI_EN            => UMI_EN,
      --
      FCLK_RD           => FCLK_RD,
      SRC_FCLK_RD       => SRC_FCLK_RD,
      --
      FCLK_WR           => FCLK_WR,
      SRC_FCLK_WR       => SRC_FCLK_WR,
      --
      FCLK_G            => FCLK_G,
      SRC_FCLK_G        => SRC_FCLK_G,
      --
      INFIFO_READ_REQ   => INFIFO_READ_REQ,
      INFIFO_DVALID     => INFIFO_DVALID,
      INFIFO_SINGLE     => INFIFO_SINGLE,
      INFIFO_BURST      => INFIFO_BURST,
      INFIFO0_D         => INFIFO0_D,
      INFIFO1_D         => INFIFO1_D,
      INFIFO2_D         => INFIFO2_D,
      INFIFO3_D         => INFIFO3_D,
      INFIFO4_D         => INFIFO4_D,
      INFIFO5_D         => INFIFO5_D,
      --
      OUTFIFO_READY     => OUTFIFO_READY,
      OUTFIFO_WRITE     => OUTFIFO_WRITE,
      OUTFIFO_D         => OUTFIFO_D,
      --
      MSG_CLK           => MSG_CLK,
      MSG_DIN           => MSG_DIN,
      MSG_ADDR          => MSG_ADDR,
      MSG_AEN           => MSG_AEN,
      MSG_WEN           => MSG_WEN,
      MSG_REN           => MSG_REN,
      MSG_DONE          => MSG_DONE,
      MSG_COUNT         => MSG_COUNT,
      MSG_CLEAR         => MSG_CLEAR,
      MSG_READY         => MSG_READY,
      MSG_SEND_MSG      => MSG_SEND_MSG,
      MSG_CE            => MSG_CE,
      MSG_DOUT          => MSG_DOUT,
      MSG_SEND_ID       => MSG_SEND_ID,
      MSG_LAST_BYTE     => MSG_LAST_BYTE,
      --
      CAM_CLK           => CAM_CLK,
      CAM_DATA          => CAM_DATA,
      CAM_CTRL          => CAM_CTRL,
      CAM_SER_TC        => CAM_SER_TC,
      CAM_SER_TFG       => CAM_SER_TFG,
      --
      CONN_A_IN         => CONN_A_IN,
      CONN_A_OUT        => CONN_A_OUT,
      CONN_A_EN         => CONN_A_EN,
      CONN_B_IN         => CONN_B_IN,
      CONN_B_OUT        => CONN_B_OUT,
      CONN_B_EN         => CONN_B_EN,
      --
      FLASH_ADDR        => FLASH_ADDR(23 downto 0),
      FLASH_IN          => FLASH_IN,
      FLASH_OUT         => FLASH_OUT,
      FLASH_EN          => FLASH_EN,
      FLASH_CE          => FLASH_CE,
      FLASH_WE          => FLASH_WE,
      FLASH_OE          => FLASH_OE,
      FLASH_RP          => FLASH_RP,
      FLASH_VPEN        => FLASH_VPEN,
      FLASH_STS         => FLASH_STS,
      --
      DDR_A_USER_CLK    => DDR_A_USER_CLK,
      --
      DDR_A_USER_RST    => DDR_A_USER_RST,
      --
      DDR_A_DATA_COUNT  => DDR_A_DATA_COUNT,
      --
      DDR_A_WR_DATA     => DDR_A_WR_DATA,
      DDR_A_WR_ADDR     => DDR_A_WR_ADDR,
      DDR_A_WR_ADDR_WEN => DDR_A_WR_ADDR_WEN,
      DDR_A_WR_ADDR_FF  => DDR_A_WR_ADDR_FF,
      DDR_A_WR_ADDR_AF  => DDR_A_WR_ADDR_AF,
      DDR_A_WR_RISE_WEN => DDR_A_WR_RISE_WEN,
      DDR_A_WR_RISE_FF  => DDR_A_WR_RISE_FF,
      DDR_A_WR_RISE_AF  => DDR_A_WR_RISE_AF,
      DDR_A_WR_FALL_WEN => DDR_A_WR_FALL_WEN,
      DDR_A_WR_FALL_FF  => DDR_A_WR_FALL_FF,
      DDR_A_WR_FALL_AF  => DDR_A_WR_FALL_AF,
      DDR_A_WR_DM_WEN   => DDR_A_WR_DM_WEN,
      DDR_A_WR_DM_FF    => DDR_A_WR_DM_FF,
      DDR_A_WR_DM_AF    => DDR_A_WR_DM_AF,
      --
      DDR_A_RD_DATA     => DDR_A_RD_DATA,
      DDR_A_RD_ADDR     => DDR_A_RD_ADDR,
      DDR_A_RD_ADDR_WEN => DDR_A_RD_ADDR_WEN,
      DDR_A_RD_ADDR_FF  => DDR_A_RD_ADDR_FF,
      DDR_A_RD_ADDR_AF  => DDR_A_RD_ADDR_AF,
      DDR_A_RD_RISE_REN => DDR_A_RD_RISE_REN,
      DDR_A_RD_RISE_EF  => DDR_A_RD_RISE_EF,
      DDR_A_RD_RISE_AE  => DDR_A_RD_RISE_AE,
      DDR_A_RD_FALL_REN => DDR_A_RD_FALL_REN,
      DDR_A_RD_FALL_EF  => DDR_A_RD_FALL_EF,
      DDR_A_RD_FALL_AE  => DDR_A_RD_FALL_AE,
      --
      DDR_B_USER_CLK    => DDR_B_USER_CLK,
      --
      DDR_B_USER_RST    => DDR_B_USER_RST,
      --
      DDR_B_DATA_COUNT  => DDR_B_DATA_COUNT,
      --
      DDR_B_WR_DATA     => DDR_B_WR_DATA,
      DDR_B_WR_ADDR     => DDR_B_WR_ADDR,
      DDR_B_WR_ADDR_WEN => DDR_B_WR_ADDR_WEN,
      DDR_B_WR_ADDR_FF  => DDR_B_WR_ADDR_FF,
      DDR_B_WR_ADDR_AF  => DDR_B_WR_ADDR_AF,
      DDR_B_WR_RISE_WEN => DDR_B_WR_RISE_WEN,
      DDR_B_WR_RISE_FF  => DDR_B_WR_RISE_FF,
      DDR_B_WR_RISE_AF  => DDR_B_WR_RISE_AF,
      DDR_B_WR_FALL_WEN => DDR_B_WR_FALL_WEN,
      DDR_B_WR_FALL_FF  => DDR_B_WR_FALL_FF,
      DDR_B_WR_FALL_AF  => DDR_B_WR_FALL_AF,
      DDR_B_WR_DM_WEN   => DDR_B_WR_DM_WEN,
      DDR_B_WR_DM_FF    => DDR_B_WR_DM_FF,
      DDR_B_WR_DM_AF    => DDR_B_WR_DM_AF,
      --
      DDR_B_RD_DATA     => DDR_B_RD_DATA,
      DDR_B_RD_ADDR     => DDR_B_RD_ADDR,
      DDR_B_RD_ADDR_WEN => DDR_B_RD_ADDR_WEN,
      DDR_B_RD_ADDR_FF  => DDR_B_RD_ADDR_FF,
      DDR_B_RD_ADDR_AF  => DDR_B_RD_ADDR_AF,
      DDR_B_RD_RISE_REN => DDR_B_RD_RISE_REN,
      DDR_B_RD_RISE_EF  => DDR_B_RD_RISE_EF,
      DDR_B_RD_RISE_AE  => DDR_B_RD_RISE_AE,
      DDR_B_RD_FALL_REN => DDR_B_RD_FALL_REN,
      DDR_B_RD_FALL_EF  => DDR_B_RD_FALL_EF,
      DDR_B_RD_FALL_AE  => DDR_B_RD_FALL_AE,
      --
      PPC_TO_DDR_IF_A   => PPC_TO_DDR_IF_A,
      DDR_TO_PPC_IF_A   => DDR_TO_PPC_IF_A,
      --
      PPC_TO_DDR_IF_B   => PPC_TO_DDR_IF_B,
      DDR_TO_PPC_IF_B   => DDR_TO_PPC_IF_B );

  iCFG : IBUF port map (I=>config, O=>xCONFIG);

  oUDP : OBUF port map (I=>xUDPRES, O=>udpres);

  oLED0 : OBUF port map (I=>xLED(0), O=>led(0));
  oLED1 : OBUF port map (I=>xLED(1), O=>led(1));
  oLED2 : OBUF port map (I=>xLED(2), O=>led(2));
  oLED3 : OBUF port map (I=>xLED(3), O=>led(3));
  oLED4 : OBUF port map (I=>xLED(4), O=>led(4));

  oBEN : OBUF port map (I=>xBOOTEN, O=>booten);

  iAFS : IBUF port map (I=>addr_flagsel, O=>xADDR_FLAGSEL);

  iCID0 : IBUF port map (I=>cid(0), O=>xCID(0));
  iCID1 : IBUF port map (I=>cid(1), O=>xCID(1));
  iCID2 : IBUF port map (I=>cid(2), O=>xCID(2));
  iCID3 : IBUF port map (I=>cid(3), O=>xCID(3));

  iMID0 : IBUF port map (I=>mid(0), O=>xMID(0));
  iMID1 : IBUF port map (I=>mid(1), O=>xMID(1));
  iMID2 : IBUF port map (I=>mid(2), O=>xMID(2));
  iMID3 : IBUF port map (I=>mid(3), O=>xMID(3));

end RTL;
