-- ==============================================================
-- FILE : USER_AP_TPL.VHD  -  MUST be modified by the USER
-- ==============================================================
--
-- This file is a TEMPLATE for the USER_AP entity for any example
-- for the HERON-FPGA9 module.
--
-- This file should be copied into a new project by the user.
--
-- ==============================================================
--
--  Date        : 01/07/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  description : User code section
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    01-07-04    First Written
--  2.0     R. Williams    16-09-05    Added PowerPC to DDR memory
--                                     interface.
--  2.1     R. Williams    09-06-06    Removed HEPC8 support.
--                                     Minor fix to HE_RD_6F logic.
--                                     Processor access sequencing
--                                     fix to HE_DDR logic.
--
-- ==============================================================



-- ==============================================================
--   Package : CONFIG
-- ==============================================================
--
-- This package allow users to set up the different clock domains
-- of the application (FIFO interfaces) in the top level.
-- Please refer to the HUNT documentation for more details.
--
-- ==============================================================

package CONFIG is

  -- FIFO CLOCK
  -- ==========

--   -----------------------------------------------------------------
--  | FCLK_G_DOMAIN |  HIGH_FCLK_G  |  HIGH_FCLK_RD  |  HIGH_FCLK_WR  |
--   -----------------------------------------------------------------
--  |     True      | True / False  |     n.a.       |     n.a.       |  single FIFO clock
--   -----------------------------------------------------------------
--  |     False     |     n.a.      |  True / False  |  True / False  |  two FIFO clocks
--   -----------------------------------------------------------------

  -- To use a single common FIFO clock, set FCLK_G_DOMAIN = TRUE
  constant FCLK_G_DOMAIN : boolean := TRUE;

  -- When read/write FIFO clock frequency > 60 MHz, set HIGH_FCLK_G = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = TRUE
  constant HIGH_FCLK_G   : boolean := TRUE;

  -- When read FIFO clock frequency > 60 MHz, set HIGH_FCLK_RD = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_RD  : boolean := FALSE;

  -- When write FIFO clock frequency > 60 MHz, set HIGH_FCLK_WR = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_WR  : boolean := FALSE;

end CONFIG;


-- ==============================================================
--   Module : USER_AP
-- ==============================================================

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;
  use IEEE.numeric_std.all;
  use WORK.CONFIG.all;


--pragma translate_off
library sim_lib;
  use sim_lib.all;

library UNISIM;
  use UNISIM.VCOMPONENTS.all;
--pragma translate_on

entity USER_AP is
  port (
    RESET             : in  std_logic;                      -- Asynchronous reset active high
    CONFIG            : in  std_logic;                      -- System wide Config signal
-- CLOCK IO
    OSC0              : in  std_logic;                      -- Xtal OSC0
    OSC3              : in  std_logic;                      -- Xtal OSC3 factory default fitted with 100Mhz
-- MISCELLANEOUS IO
    UDPRES            : out std_logic;                      -- reset the carrier (high if NOT used)
    LED               : out std_logic_vector(4 downto 0);   -- LEDs
    ADDR_FLAGSEL      : in  std_logic;
    BOOTEN            : out std_logic;
    CID               : in  std_logic_vector(3 downto 0);   -- Carrier ID
    MID               : in  std_logic_vector(3 downto 0);   -- Module ID
-- UMIs
    UMI_IN            : in  std_logic_vector(3 downto 0);
    UMI_OUT           : out std_logic_vector(3 downto 0);
    UMI_EN            : out std_logic_vector(3 downto 0);
-- READ FIFO CLK
    FCLK_RD           : in  std_logic;                      -- FIFO_IN clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_RD       : out std_logic;                      -- FIFO_IN clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- WRITE FIFO CLK
    FCLK_WR           : in  std_logic;                      -- FIFO_OUT clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_WR       : out std_logic;                      -- FIFO_OUT clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- COMMON FIFO CLK
    FCLK_G            : in  std_logic;                      -- Common FIFO clock to be used in this module (only when FCLK_G_DOMAIN=TRUE)
    SRC_FCLK_G        : out std_logic;                      -- Common FIFO clock source for the top level  (only when FCLK_G_DOMAIN=TRUE)
-- FIFO IN / HE_RD_6F interface
    INFIFO_READ_REQ   : out std_logic_vector(5 downto 0);   -- FIFO_IN Read Request
    INFIFO_DVALID     : in  std_logic_vector(5 downto 0);   -- FIFO_IN Data Valid
    INFIFO_SINGLE     : in  std_logic_vector(5 downto 0);   -- FIFO_IN Single data item available
    INFIFO_BURST      : in  std_logic_vector(5 downto 0);   -- FIFO_IN Block of data available
    INFIFO0_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 0
    INFIFO1_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 1
    INFIFO2_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 2
    INFIFO3_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 3
    INFIFO4_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 4
    INFIFO5_D         : in  std_logic_vector(31 downto 0);  -- FIFO_IN Data for FIFO 5
-- FIFO OUT / HE_WR_6F interface
    OUTFIFO_READY     : in  std_logic_vector(5 downto 0);   -- FIFO_OUT Ready Flag
    OUTFIFO_WRITE     : out std_logic_vector(5 downto 0);   -- FIFO_OUT Write Command
    OUTFIFO_D         : out std_logic_vector(31 downto 0);  -- FIFO_OUT Data
-- SERIAL BUS / HE_USER Message interface
    MSG_CLK           : out std_logic;                      -- HSB clock source for the top level
    MSG_DIN           : in  std_logic_vector(7 downto 0);   -- data received from HSB
    MSG_ADDR          : in  std_logic_vector(7 downto 0);   -- HSB address
    MSG_AEN           : in  std_logic;                      -- address enable
    MSG_WEN           : in  std_logic;                      -- write access
    MSG_REN           : in  std_logic;                      -- read access
    MSG_DONE          : in  std_logic;                      -- message successfully transmitted
    MSG_COUNT         : in  std_logic;                      -- user's counter enable
    MSG_CLEAR         : in  std_logic;                      -- user's counter clear
    MSG_READY         : out std_logic;                      -- to acknowledge an access
    MSG_SEND_MSG      : out std_logic;                      -- message send command
    MSG_CE            : out std_logic;                      -- to control speed operation
    MSG_DOUT          : out std_logic_vector(7 downto 0);   -- data sent to HSB
    MSG_SEND_ID       : out std_logic;                      -- ID information send command
    MSG_LAST_BYTE     : out std_logic;                      -- to indicate when the last byte is presented
-- CONNECTOR DIGITAL IO
    CONN_A_IN         : in  std_logic_vector(14 downto 0);  -- Data input on Connector A
    CONN_A_OUT        : out std_logic_vector(14 downto 0);  -- Data output on Connector A
    CONN_A_EN         : out std_logic_vector(14 downto 0);  -- Output enables for Connector A
    CONN_B_IN         : in  std_logic_vector(14 downto 0);  -- Data input on Connector B
    CONN_B_OUT        : out std_logic_vector(14 downto 0);  -- Data output on Connector B
    CONN_B_EN         : out std_logic_vector(14 downto 0);  -- Output enables for Connector B
-- FLASH interface
    FLASH_ADDR        : out std_logic_vector(23 downto 0) ; -- FLASH Address
    FLASH_IN          : in  std_logic_vector(7 downto 0);   -- FLASH Data-In
    FLASH_OUT         : out std_logic_vector(7 downto 0);   -- FLASH Data-Out
    FLASH_EN          : out std_logic_vector(7 downto 0);   -- FLASH Data-Out Enable
    FLASH_CE          : out std_logic;                      -- FLASH Chip Enable
    FLASH_WE          : out std_logic;                      -- FLASH Write Enable
    FLASH_OE          : out std_logic;                      -- FLASH Output Enable
    FLASH_RP          : out std_logic;                      -- FLASH Reset/Power-Down
    FLASH_VPEN        : out std_logic;                      -- FLASH Program Enable
    FLASH_STS         : in  std_logic;                      -- FLASH Status
-- DDR User Clock A
    DDR_A_USER_CLK    : out std_logic;                      -- DDR Port A Clock source
-- DDR User Reset A
    DDR_A_USER_RST    : out std_logic;                      -- DDR Port A Data Count Reset
-- DDR Data Count
    DDR_A_DATA_COUNT  : in  std_logic_vector(31 downto 0);  -- DDR Port A Data Count
-- DDR Write Port A
    DDR_A_WR_DATA     : out std_logic_vector(71 downto 0);  -- DDR Port A Write Data
    DDR_A_WR_ADDR     : out std_logic_vector(24 downto 0);  -- DDR Port A Write Address
    DDR_A_WR_ADDR_WEN : out std_logic;                      -- DDR Port A Wr-Address Enable
    DDR_A_WR_ADDR_FF  : in  std_logic;                      -- DDR Port A Wr-Address Full Flag
    DDR_A_WR_ADDR_AF  : in  std_logic;                      -- DDR Port A Wr-Address Almost Full
    DDR_A_WR_RISE_WEN : out std_logic;                      -- DDR Port A Rising Data Enable
    DDR_A_WR_RISE_FF  : in  std_logic;                      -- DDR Port A Rising Data Full Flag
    DDR_A_WR_RISE_AF  : in  std_logic;                      -- DDR Port A Rising Data Almost Full
    DDR_A_WR_FALL_WEN : out std_logic;                      -- DDR Port A Falling Data Enable
    DDR_A_WR_FALL_FF  : in  std_logic;                      -- DDR Port A Falling Data Full Flag
    DDR_A_WR_FALL_AF  : in  std_logic;                      -- DDR Port A Falling Data Almost Full
    DDR_A_WR_DM_WEN   : out std_logic;                      -- DDR Port A Data Mask Enable
    DDR_A_WR_DM_FF    : in  std_logic;                      -- DDR Port A Data Mask Full Flag
    DDR_A_WR_DM_AF    : in  std_logic;                      -- DDR Port A Data Mask Almost Full
-- DDR Read Port A
    DDR_A_RD_DATA     : in  std_logic_vector(63 downto 0) ; -- DDR Port A Read Data
    DDR_A_RD_ADDR     : out std_logic_vector(24 downto 0) ; -- DDR Port A Read Address
    DDR_A_RD_ADDR_WEN : out std_logic;                      -- DDR Port A Rd-Address Enable
    DDR_A_RD_ADDR_FF  : in  std_logic;                      -- DDR Port A Rd-Address Full Flag
    DDR_A_RD_ADDR_AF  : in  std_logic;                      -- DDR Port A Rd-Address Almost Full
    DDR_A_RD_RISE_REN : out std_logic;                      -- DDR Port A Rising Data Enable
    DDR_A_RD_RISE_EF  : in  std_logic;                      -- DDR Port A Rising Data Empty Flag
    DDR_A_RD_RISE_AE  : in  std_logic;                      -- DDR Port A Rising Data Almost Empty
    DDR_A_RD_FALL_REN : out std_logic;                      -- DDR Port A Falling Data Enable
    DDR_A_RD_FALL_EF  : in  std_logic;                      -- DDR Port A Falling Data Empty Flag
    DDR_A_RD_FALL_AE  : in  std_logic;                      -- DDR Port A Falling Data Almost Empty
-- DDR User Clock B
    DDR_B_USER_CLK    : out std_logic;                      -- DDR Port B Clock source
-- DDR User Reset B
    DDR_B_USER_RST    : out std_logic;                      -- DDR Port B Data Count Reset
-- DDR Data Count
    DDR_B_DATA_COUNT  : in  std_logic_vector(31 downto 0);  -- DDR Port B Data Count
-- DDR Write Port B
    DDR_B_WR_DATA     : out std_logic_vector(71 downto 0);  -- DDR Port B Write Data
    DDR_B_WR_ADDR     : out std_logic_vector(24 downto 0);  -- DDR Port B Write Address
    DDR_B_WR_ADDR_WEN : out std_logic;                      -- DDR Port B Wr-Address Enable
    DDR_B_WR_ADDR_FF  : in  std_logic;                      -- DDR Port B Wr-Address Full Flag
    DDR_B_WR_ADDR_AF  : in  std_logic;                      -- DDR Port B Wr-Address Almost Full
    DDR_B_WR_RISE_WEN : out std_logic;                      -- DDR Port B Rising Data Enable
    DDR_B_WR_RISE_FF  : in  std_logic;                      -- DDR Port B Rising Data Full Flag
    DDR_B_WR_RISE_AF  : in  std_logic;                      -- DDR Port B Rising Data Almost Full
    DDR_B_WR_FALL_WEN : out std_logic;                      -- DDR Port B Falling Data Enable
    DDR_B_WR_FALL_FF  : in  std_logic;                      -- DDR Port B Falling Data Full Flag
    DDR_B_WR_FALL_AF  : in  std_logic;                      -- DDR Port B Falling Data Almost Full
    DDR_B_WR_DM_WEN   : out std_logic;                      -- DDR Port B Data Mask Enable
    DDR_B_WR_DM_FF    : in  std_logic;                      -- DDR Port B Data Mask Full Flag
    DDR_B_WR_DM_AF    : in  std_logic;                      -- DDR Port B Data Mask Almost Full
-- DDR Read Port B
    DDR_B_RD_DATA     : in  std_logic_vector(63 downto 0);  -- DDR Port B Read Data
    DDR_B_RD_ADDR     : out std_logic_vector(24 downto 0);  -- DDR Port B Read Address
    DDR_B_RD_ADDR_WEN : out std_logic;                      -- DDR Port B Rd-Address Enable
    DDR_B_RD_ADDR_FF  : in  std_logic;                      -- DDR Port B Rd-Address Full Flag
    DDR_B_RD_ADDR_AF  : in  std_logic;                      -- DDR Port B Rd-Address Almost Full
    DDR_B_RD_RISE_REN : out std_logic;                      -- DDR Port B Rising Data Enable
    DDR_B_RD_RISE_EF  : in  std_logic;                      -- DDR Port B Rising Data Empty Flag
    DDR_B_RD_RISE_AE  : in  std_logic;                      -- DDR Port B Rising Data Almost Empty
    DDR_B_RD_FALL_REN : out std_logic;                      -- DDR Port B Falling Data Enable
    DDR_B_RD_FALL_EF  : in  std_logic;                      -- DDR Port B Falling Data Empty Flag
    DDR_B_RD_FALL_AE  : in  std_logic;                      -- DDR Port B Falling Data Almost Empty
-- DDR PPC Interface for Port A
    PPC_TO_DDR_IF_A   : out std_logic_vector(130 downto 0); -- PowerPC to DDR Port A interface signals
    DDR_TO_PPC_IF_A   : in  std_logic_vector(77 downto 0);  -- DDR to PowerPC Port A interface signals
-- DDR PPC Interface for Port B
    PPC_TO_DDR_IF_B   : out std_logic_vector(130 downto 0); -- PowerPC to DDR Port B interface signals
    DDR_TO_PPC_IF_B   : in  std_logic_vector(77 downto 0)   -- DDR to PowerPC Port B interface signals
  );
end USER_AP;


architecture TEMPLATE of USER_AP is

  signal MCLK : std_logic;  -- main clock

begin


  -- ==========================================================
  -- ==========================================================
  --       <<<<<<<   INSERT YOUR CODE HERE    >>>>>>>>>
  -- ==========================================================
  -- ==========================================================


  -------------------------------------------------------------
  -- OSC3 clock divided by 2

  MCLK <= OSC3;

  -------------------------------------------------------------
  -- Drive the UDP-RESET high (inactive)

  UDPRES <= '1';

  -------------------------------------------------------------
  -- Drive the BOOT ENABLE signal high (inactive)

  BOOTEN <= '1';

  -------------------------------------------------------------
  -- Drive unused signals

  UMI_OUT <= (others=>'0');
  UMI_EN  <= (others=>'1');

  -------------------------------------------------------------
  -- FIFOs

  SRC_FCLK_RD <= '0';   -- clock source for READ FIFO
  SRC_FCLK_WR <= '0';   -- clock source for WRITE FIFO
  SRC_FCLK_G  <= MCLK;  -- global clock source for both FIFOs

  -------------------------------------------------------------
  -- DDR SDRAM

  DDR_A_USER_CLK    <= '0';
  DDR_A_USER_RST    <= '0';

  DDR_A_WR_DATA     <= (others=>'0');
  DDR_A_WR_ADDR     <= (others=>'0');
  DDR_A_WR_ADDR_WEN <= '0';
  DDR_A_WR_RISE_WEN <= '0';
  DDR_A_WR_FALL_WEN <= '0';
  DDR_A_WR_DM_WEN   <= '0';

  DDR_A_RD_ADDR     <= (others=>'0');
  DDR_A_RD_ADDR_WEN <= '0';
  DDR_A_RD_RISE_REN <= '0';
  DDR_A_RD_FALL_REN <= '0';

  DDR_B_USER_CLK    <= '0';
  DDR_B_USER_RST    <= '0';

  DDR_B_WR_DATA     <= (others=>'0');
  DDR_B_WR_ADDR     <= (others=>'0');
  DDR_B_WR_ADDR_WEN <= '0';
  DDR_B_WR_RISE_WEN <= '0';
  DDR_B_WR_FALL_WEN <= '0';
  DDR_B_WR_DM_WEN   <= '0';

  DDR_B_RD_ADDR     <= (others=>'0');
  DDR_B_RD_ADDR_WEN <= '0';
  DDR_B_RD_RISE_REN <= '0';
  DDR_B_RD_FALL_REN <= '0';

  -------------------------------------------------------------
  -- PowerPC to DDR Interface

  PPC_TO_DDR_IF_A <= (others=>'0');
  PPC_TO_DDR_IF_B <= (others=>'0');

  -------------------------------------------------------------
  -- Serial Bus

  MSG_CLK <= OSC3;  -- clock for Message Interface

  -------------------------------------------------------------
  -- Connector Digital IO

  CONN_A_OUT <= (others=>'0');
  CONN_A_EN  <= (others=>'1');

  CONN_B_OUT <= (others=>'0');
  CONN_B_EN  <= (others=>'1');

  -------------------------------------------------------------
  -- FLASH memory interface

  FLASH_ADDR <= (others=>'0');

  FLASH_OUT  <= (others=>'0');
  FLASH_EN   <= (others=>'1');

  FLASH_CE   <= '1';
  FLASH_WE   <= '1';
  FLASH_OE   <= '1';

  FLASH_RP   <= '0';
  FLASH_VPEN <= '0';

end TEMPLATE;
