-- ==============================================================
-- FILE : HSB1.VHD  -  for modification by the USER
-- ==============================================================
--
-- This file is part of the PPC-DDR Example, for the HERON-FPGA9
-- module.
--
-- This file should be modified by the users to implement their
-- own logic.
--
-- ==============================================================
--
--  Module      : HSB1
--  Date        : 01/07/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : Using HUNT Message Interface
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    01-07-04    First Written
--  2.0     R. Williams    16-09-05    Added PowerPC to DDR memory
--                                     interface.
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;

entity HSB1 is
  port (
    RST            : in  std_logic;
    CLK            : in  std_logic;
    DIN            : in  std_logic_vector(7 downto 0);
    ADDR           : in  std_logic_vector(7 downto 0);
    AEN            : in  std_logic;
    WEN            : in  std_logic;
    REN            : in  std_logic;
    DONE           : in  std_logic;
    COUNT          : in  std_logic;
    CLEAR          : in  std_logic;
    READY          : out std_logic;
    SEND_MSG       : out std_logic;
    CE             : out std_logic;
    DOUT           : out std_logic_vector(7 downto 0);
    SEND_ID        : out std_logic;
    LAST_BYTE      : out std_logic;
    --
    HSB_IP_TX_WEN  : out std_logic;
    HSB_IP_TX_DATA : out std_logic_vector(7 downto 0);
    HSB_IP_RX_REN  : out std_logic;
    HSB_IP_RX_DATA : in  std_logic_vector(7 downto 0);
    HSB_IP_COUNT   : in  std_logic_vector(7 downto 0);
    HSB_IP_FLAGS   : in  std_logic_vector(7 downto 0)
  );
end HSB1;


architecture RTL of HSB1 is

  signal REG_0 : std_logic_vector(7 downto 0) := "00000000";
  signal REG_1 : std_logic_vector(7 downto 0) := (others=>'0');

  signal EN : std_logic_vector(1 downto 0);

  signal HSB_DATA_ACCESS   : std_logic;
  signal HSB_CTRL_ACCESS   : std_logic;

  signal REN_DELAY    : std_logic;
  signal WEN_DELAY    : std_logic;
  signal SINGLE_REN   : std_logic;
  signal SINGLE_WEN   : std_logic;

  signal READYc       : std_logic;
  signal RD_DONE      : std_logic;

begin

  SEND_MSG  <= '0';
  CE        <= '0';
  SEND_ID   <= '0';
  LAST_BYTE <= '0';

  -- Decode address & output mux
  process(ADDR, REG_0, REG_1)
  begin
    EN <= (others=>'0');
    case ADDR is
      when "00000000" => EN(0)<='1';
      when "00000001" => EN(1)<='1';
      when others     => NULL;
    end case;
  end process;

  process(ADDR, HSB_IP_RX_DATA, HSB_IP_COUNT, HSB_IP_FLAGS)
  begin
    case ADDR is
      when "00000000" => DOUT <= HSB_IP_RX_DATA;
      when "00000001" => DOUT <= HSB_IP_COUNT;
      when "00000011" => DOUT <= HSB_IP_FLAGS;
      when others     => DOUT <= (others=>'-');
    end case;
  end process;

  -- DATA_REGs
  process (CLK)
  begin
    if rising_edge(CLK) then
      if WEN='1' then
        if EN(0)='1' then
          REG_0 <= DIN;
        end if;
        if EN(1)='1' then
          REG_1 <= DIN;
        end if;
      end if;
    end if;
  end process;

  -- READY
  READYc <= WEN OR RD_DONE;

  process (RST, CLK)
  begin
    if RST='1' then
      READY <= '1';
    elsif rising_edge(CLK) then
      READY <= READYc;
    end if;
  end process;

  process (RST, CLK)
  begin
    if RST='1' then
      REN_DELAY  <= '0';
      WEN_DELAY  <= '0';
      SINGLE_REN <= '0';
      SINGLE_WEN <= '0';
      RD_DONE    <= '0';
    elsif rising_edge(CLK) then
      REN_DELAY  <= REN;
      WEN_DELAY  <= WEN;
      SINGLE_REN <= '0';
      SINGLE_WEN <= '0';
      if REN_DELAY='0' and REN='1' then
        SINGLE_REN <= '1';
      end if;
      if WEN_DELAY='1' and WEN='0' then
        SINGLE_WEN <= '1';
      end if;
      RD_DONE <= '0';
      if (SINGLE_REN='1' or (RD_DONE='1' and REN='1')) then
        RD_DONE <= '1';
      end if;
    end if;
  end process;

  HSB_IP_RX_REN  <= HSB_DATA_ACCESS AND SINGLE_REN;
  HSB_IP_TX_WEN  <= HSB_DATA_ACCESS AND SINGLE_WEN;

  -- Decode address & output mux
  process(ADDR)
  begin
    HSB_DATA_ACCESS  <= '0';
    HSB_CTRL_ACCESS  <= '0';
    case ADDR is
      when "00000000" => HSB_DATA_ACCESS <= '1';
      when "00000001" => HSB_CTRL_ACCESS <= '1';
      when "00000011" => HSB_CTRL_ACCESS <= '1';
      when others     => null;
    end case;
  end process;

  HSB_IP_TX_DATA  <= DIN;

end RTL;
