--------------------------------------------------------------------------------
-- FILE : USER_LOGIC.VHD
--------------------------------------------------------------------------------
--
-- This file is part of the DDR Example for the PowerPC.
--
--------------------------------------------------------------------------------
--
--  Date        : 17/09/2007
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : OPB to HSB interface
--
--------------------------------------------------------------------------------
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    17-09-07    First Written for EDK9.1, based on Xilinx
--                                     example code - OPB GPIO v3.01.a.
--
--------------------------------------------------------------------------------
--
--  Structure:
--
--       opb2hsb.vhd
--          \ opb_ipif_ssp1.vhd
--          \ user_logic.vhd
--
-------------------------------------------------------------------------------


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.std_logic_signed.all;
  use IEEE.std_logic_misc.all;


library Unisim;
  use Unisim.all;


entity user_logic is
  port (
    Bus2IP_Addr  : in  std_logic_vector(0 to 31);
    Bus2IP_Clk   : in  std_logic;
    Bus2IP_CS    : in  std_logic;
    Bus2IP_Data  : in  std_logic_vector(0 to 31);
    Bus2IP_RdCE  : in  std_logic;
    Bus2IP_Reset : in  std_logic;
    Bus2IP_WrCE  : in  std_logic;
    IP2Bus_Ack   : out std_logic;
    IP2Bus_Data  : out std_logic_vector(0 to 31);
    IP_CLOCK     : in  std_logic;
    IP_RESET     : in  std_logic;
    IP_TX_WEN    : in  std_logic;
    IP_TX_DATA   : in  std_logic_vector(7 downto 0);
    IP_RX_REN    : in  std_logic;
    IP_RX_DATA   : out std_logic_vector(7 downto 0);
    IP_COUNT     : out std_logic_vector(7 downto 0);
    IP_FLAGS     : out std_logic_vector(7 downto 0)
  );
end entity user_logic;


architecture RTL of user_logic is

  component fifo15x8
    port (
      din          : in  std_logic_vector(7 downto 0);
      wr_en        : in  std_logic;
      wr_clk       : in  std_logic;
      rd_en        : in  std_logic;
      rd_clk       : in  std_logic;
      ainit        : in  std_logic;
      dout         : out std_logic_vector(7 downto 0);
      full         : out std_logic;
      empty        : out std_logic;
      almost_full  : out std_logic;
      almost_empty : out std_logic;
      wr_count     : out std_logic_vector(3 downto 0);
      rd_count     : out std_logic_vector(3 downto 0)
    );
  end component;

signal RdCE_rg          : std_logic;
signal DATA_ACCESS      : std_logic;

signal addr             : std_logic_vector(0 to 1);
signal count_reg        : std_logic_vector(0 to 7);
signal flags_reg        : std_logic_vector(0 to 7);

signal PPC2HSB_RST      : std_logic;
signal HSB2PPC_RST      : std_logic;

signal PPC2HSB_WEN      : std_logic;
signal PPC2HSB_REN      : std_logic;

signal HSB2PPC_WEN      : std_logic;
signal HSB2PPC_REN      : std_logic;

signal PPC2HSB_FF       : std_logic;
signal PPC2HSB_EF       : std_logic;
signal PPC2HSB_AF       : std_logic;
signal PPC2HSB_AE       : std_logic;

signal PPC2HSB_FF_RG1   : std_logic;
signal PPC2HSB_FF_RG2   : std_logic;
signal PPC2HSB_AF_RG1   : std_logic;
signal PPC2HSB_AF_RG2   : std_logic;

signal PPC2HSB_EF_RG1   : std_logic;
signal PPC2HSB_EF_RG2   : std_logic;
signal PPC2HSB_AE_RG1   : std_logic;
signal PPC2HSB_AE_RG2   : std_logic;

signal HSB2PPC_FF       : std_logic;
signal HSB2PPC_EF       : std_logic;
signal HSB2PPC_AF       : std_logic;
signal HSB2PPC_AE       : std_logic;

signal HSB2PPC_EF_RG1   : std_logic;
signal HSB2PPC_EF_RG2   : std_logic;
signal HSB2PPC_AE_RG1   : std_logic;
signal HSB2PPC_AE_RG2   : std_logic;

signal HSB2PPC_FF_RG1   : std_logic;
signal HSB2PPC_FF_RG2   : std_logic;
signal HSB2PPC_AF_RG1   : std_logic;
signal HSB2PPC_AF_RG2   : std_logic;

signal PPC2HSB_DATA     : std_logic_vector(7 downto 0);
signal HSB2PPC_DATA     : std_logic_vector(0 to 7);

signal PPC2HSB_WR_COUNT : std_logic_vector(3 downto 0);
signal PPC2HSB_RD_COUNT : std_logic_vector(3 downto 0);

signal HSB2PPC_WR_COUNT : std_logic_vector(3 downto 0);
signal HSB2PPC_RD_COUNT : std_logic_vector(3 downto 0);

begin

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      RdCE_rg <= Bus2IP_RdCE;
    end if;
  end process;

  IP2Bus_Ack <= Bus2IP_WrCE or RdCE_rg;

  -- OPB Address - Big Endian
  addr <= Bus2IP_Addr(28 to 29);

  IP2Bus_Data(0 to 23) <= (others => '0');

  DATA_ACCESS <= '1' when addr="00" else '0';

  process (HSB2PPC_DATA, count_reg, flags_reg) is
  begin
    case addr is
      when "00"   => IP2Bus_Data(24 to 31) <= HSB2PPC_DATA;
      when "01"   => IP2Bus_Data(24 to 31) <= count_reg;
      when "10"   => IP2Bus_Data(24 to 31) <= flags_reg;
      when others => IP2Bus_Data(24 to 31) <= (others=>'0');
    end case;
  end process;

  count_reg(0 to 3) <= HSB2PPC_RD_COUNT(3 downto 0);
  count_reg(4 to 7) <= PPC2HSB_WR_COUNT(3 downto 0);

  flags_reg(7) <= PPC2HSB_FF;
  flags_reg(6) <= PPC2HSB_AF;
  flags_reg(5) <= PPC2HSB_AE_RG2;
  flags_reg(4) <= PPC2HSB_EF_RG2;
  flags_reg(3) <= HSB2PPC_EF;
  flags_reg(2) <= HSB2PPC_AE;
  flags_reg(1) <= HSB2PPC_AF_RG2;
  flags_reg(0) <= HSB2PPC_FF_RG2;

--------------------------------
-- PPC2HSB FIFO connection

  PPC2HSB_RST <= Bus2IP_Reset OR IP_RESET;

  PPC2HSB_WEN <= Bus2IP_CS AND Bus2IP_WrCE AND DATA_ACCESS;

  PPC2HSB_REN <= IP_RX_REN;

  iPPC2HSB_FIFO : fifo15x8
    port map (
      din          => Bus2IP_Data(24 to 31),
      wr_en        => PPC2HSB_WEN,
      wr_clk       => Bus2IP_Clk,
      rd_en        => PPC2HSB_REN,
      rd_clk       => IP_CLOCK,
      ainit        => PPC2HSB_RST,
      dout         => PPC2HSB_DATA,
      full         => PPC2HSB_FF,
      empty        => PPC2HSB_EF,
      almost_full  => PPC2HSB_AF,
      almost_empty => PPC2HSB_AE,
      wr_count     => PPC2HSB_WR_COUNT,
      rd_count     => PPC2HSB_RD_COUNT );

  process(IP_CLOCK) is
  begin
    if IP_CLOCK'event and IP_CLOCK='1' then
      PPC2HSB_FF_RG1 <= PPC2HSB_FF;
      PPC2HSB_AF_RG1 <= PPC2HSB_AF;
      PPC2HSB_FF_RG2 <= PPC2HSB_FF_RG1;
      PPC2HSB_AF_RG2 <= PPC2HSB_AF_RG1;
    end if;
  end process;

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      PPC2HSB_EF_RG1 <= PPC2HSB_EF;
      PPC2HSB_AE_RG1 <= PPC2HSB_AE;
      PPC2HSB_EF_RG2 <= PPC2HSB_EF_RG1;
      PPC2HSB_AE_RG2 <= PPC2HSB_AE_RG1;
    end if;
  end process;

  IP_FLAGS(4) <= PPC2HSB_EF;
  IP_FLAGS(5) <= PPC2HSB_AE;
  IP_FLAGS(6) <= PPC2HSB_AF_RG2;
  IP_FLAGS(7) <= PPC2HSB_FF_RG2;

  IP_COUNT(7 downto 4) <= PPC2HSB_RD_COUNT(3 downto 0);

  IP_RX_DATA <= PPC2HSB_DATA;

--------------------------------
-- HSB2PPC FIFO connection

  HSB2PPC_RST <= Bus2IP_Reset OR IP_RESET;

  HSB2PPC_WEN <= IP_TX_WEN;

  HSB2PPC_REN <= Bus2IP_CS AND Bus2IP_RdCE AND (NOT RdCE_rg) AND DATA_ACCESS;

  iHSB2PPC_FIFO : fifo15x8
    port map (
      din          => IP_TX_DATA,
      wr_en        => HSB2PPC_WEN,
      wr_clk       => IP_CLOCK,
      rd_en        => HSB2PPC_REN,
      rd_clk       => Bus2IP_Clk,
      ainit        => HSB2PPC_RST,
      dout         => HSB2PPC_DATA,
      full         => HSB2PPC_FF,
      empty        => HSB2PPC_EF,
      almost_full  => HSB2PPC_AF,
      almost_empty => HSB2PPC_AE,
      wr_count     => HSB2PPC_WR_COUNT,
      rd_count     => HSB2PPC_RD_COUNT );

  process(IP_CLOCK) is
  begin
    if IP_CLOCK'event and IP_CLOCK='1' then
      HSB2PPC_EF_RG1 <= HSB2PPC_EF;
      HSB2PPC_AE_RG1 <= HSB2PPC_AE;
      HSB2PPC_EF_RG2 <= HSB2PPC_EF_RG1;
      HSB2PPC_AE_RG2 <= HSB2PPC_AE_RG1;
    end if;
  end process;

  process(Bus2IP_Clk) is
  begin
    if Bus2IP_Clk'event and Bus2IP_Clk='1' then
      HSB2PPC_FF_RG1 <= HSB2PPC_FF;
      HSB2PPC_AF_RG1 <= HSB2PPC_AF;
      HSB2PPC_FF_RG2 <= HSB2PPC_FF_RG1;
      HSB2PPC_AF_RG2 <= HSB2PPC_AF_RG1;
    end if;
  end process;

  IP_FLAGS(0) <= HSB2PPC_FF;
  IP_FLAGS(1) <= HSB2PPC_AF;
  IP_FLAGS(2) <= HSB2PPC_AE_RG2;
  IP_FLAGS(3) <= HSB2PPC_EF_RG2;

  IP_COUNT(3 downto 0) <= HSB2PPC_WR_COUNT(3 downto 0);

end architecture RTL;
