-- ==============================================================
-- FILE : PLB2DDR.VHD
-- ==============================================================
--
-- This file is part of the PLB Power PC to DDR interface for the
-- HERON-FPGA9.
--
-- This file is derived from Xilinx provided example code.
--
-- ==============================================================
--
--  Date        : 16/09/2005
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : PLB to DDR interface
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    16-09-05    First Written
--
-- ==============================================================


library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.std_logic_unsigned.all;

library proc_common_v2_00_a;
  use proc_common_v2_00_a.proc_common_pkg.all;
  use proc_common_v2_00_a.ipif_pkg.all;
  use proc_common_v2_00_a.family.all;

library plb_ipif_v1_00_f;
  use plb_ipif_v1_00_f.all;


entity plb2ddr is
  generic (
    -- sdram generics
    C_INCLUDE_BURST_CACHELN_SUPPORT : integer := 1;
    -- auto-calculated generic
    C_PLB_MID_WIDTH                 : integer := 2;
    -- ipif generics
    C_PLB_CLK_PERIOD_PS             : integer          := 10000;
    C_FAMILY                        : string           := "virtex2p";
    C_BASEADDR                      : std_logic_vector := X"FFFF_FFFF";
    C_HIGHADDR                      : std_logic_vector := X"0000_0000";
    C_PLB_NUM_MASTERS               : integer          := 4;
    C_PLB_AWIDTH                    : integer          := 32;
    C_PLB_DWIDTH                    : integer          := 64
  );
  port (
    -- Clocks and reset
    PLB_Clk        : in  std_logic;
    SDRAM_Clk_in   : in  std_logic;
    PLB_Rst        : in  std_logic;
    -- PLB Slave signals
    PLB_ABus       : in  std_logic_vector(0 to C_PLB_AWIDTH-1);
    PLB_PAValid    : in  std_logic;
    PLB_SAValid    : in  std_logic;
    PLB_rdPrim     : in  std_logic;
    PLB_wrPrim     : in  std_logic;
    PLB_masterID   : in  std_logic_vector(0 to C_PLB_MID_WIDTH-1);
    PLB_abort      : in  std_logic;
    PLB_busLock    : in  std_logic;
    PLB_RNW        : in  std_logic;
    PLB_BE         : in  std_logic_vector(0 to (C_PLB_DWIDTH/8)-1);
    PLB_MSize      : in  std_logic_vector(0 to 1);
    PLB_size       : in  std_logic_vector(0 to 3);
    PLB_type       : in  std_logic_vector(0 to 2);
    PLB_compress   : in  std_logic;
    PLB_guarded    : in  std_logic;
    PLB_ordered    : in  std_logic;
    PLB_lockErr    : in  std_logic;
    PLB_wrDBus     : in  std_logic_vector(0 to C_PLB_DWIDTH-1);
    PLB_wrBurst    : in  std_logic;
    PLB_rdBurst    : in  std_logic;
    PLB_pendReq    : in  std_logic;
    PLB_pendPri    : in  std_logic_vector(0 to 1);
    PLB_reqPri     : in  std_logic_vector(0 to 1);
    Sl_addrAck     : out std_logic;
    Sl_SSize       : out std_logic_vector(0 to 1);
    Sl_wait        : out std_logic;
    Sl_rearbitrate : out std_logic;
    Sl_wrDAck      : out std_logic;
    Sl_wrComp      : out std_logic;
    Sl_wrBTerm     : out std_logic;
    Sl_rdDBus      : out std_logic_vector(0 to C_PLB_DWIDTH-1);
    Sl_rdWdAddr    : out std_logic_vector(0 to 3);
    Sl_rdDAck      : out std_logic;
    Sl_rdComp      : out std_logic;
    Sl_rdBTerm     : out std_logic;
    Sl_MBusy       : out std_logic_vector(0 to C_PLB_NUM_MASTERS-1);
    Sl_MErr        : out std_logic_vector(0 to C_PLB_NUM_MASTERS-1);
    -- DDR interface signals
    PPC_TO_DDR_IF  : out std_logic_vector(130 downto 0);
    DDR_TO_PPC_IF  : in  std_logic_vector(77 downto 0)
  );

  --fan-out attributes for XST
  attribute MAX_FANOUT            : string;
  attribute MAX_FANOUT of PLB_Clk : signal is "10000";
  attribute MAX_FANOUT of PLB_Rst : signal is "10000";

  attribute MIN_SIZE   : string;
  attribute MIN_SIZE of C_BASEADDR : constant is "0x08";

  attribute SIGIS            : string;
  attribute SIGIS of PLB_Clk : signal is "Clk";
  attribute SIGIS of PLB_Rst : signal is "Rst";

  attribute XRANGE                                    : string;
  attribute XRANGE of C_INCLUDE_BURST_CACHELN_SUPPORT : constant is "(0,1)";

  attribute ADDR_TYPE               : string;
  attribute ADDR_TYPE of C_BASEADDR : constant is "MEMORY";
  attribute ADDR_TYPE of C_HIGHADDR : constant is "MEMORY";

  attribute ASSIGNMENT                 : string;
  attribute ASSIGNMENT of C_BASEADDR   : constant is "REQUIRE";
  attribute ASSIGNMENT of C_PLB_AWIDTH : constant is "CONSTANT";
  attribute ASSIGNMENT of C_PLB_DWIDTH : constant is "CONSTANT";

end plb2ddr;


architecture implementation of plb2ddr is

  function bool2int(x: boolean) return integer is
  begin
    if x then
      return 1;
    else
      return 0;
    end if;
  end function bool2int;

  constant OPB_BUS        : integer := 0;
  constant PLB_BUS        : integer := 1;

  -- addresses are now expected to be 64-bits wide - create constants to 
  -- zero the most significant address bits
  constant ZERO_ADDR_PAD       : std_logic_vector(0 to 64-C_PLB_AWIDTH-1) := (others => '0');
  constant SDRAM               : integer := 120;

  constant ARD_ID_ARRAY : INTEGER_ARRAY_TYPE :=
           (
            0 => SDRAM   -- SDRAM
           );

  constant ARD_ADDR_RANGE_ARRAY  : SLV64_ARRAY_TYPE :=
           (
            ZERO_ADDR_PAD & C_BASEADDR, -- SDRAM base addr
            ZERO_ADDR_PAD & C_HIGHADDR  -- SDRAM high addr
           );

  constant ARD_DWIDTH_ARRAY     : INTEGER_ARRAY_TYPE :=
           (
            0 => C_PLB_DWIDTH   --  SDRAM data width
           );

  constant ARD_NUM_CE_ARRAY   : INTEGER_ARRAY_TYPE :=
           (
            0 => 1    -- SDRAM CE number
           );

  -- don't include MIR
  constant DEV_MIR_ENABLE       : boolean := false;
  constant DEV_BLK_ID           : integer := 1;

  -- set IPIF timeout counter to 64
  constant DEV_DPHASE_TIMEOUT   : integer := 64;

  -- set IPIF burst page size
  constant DEV_BURST_ENABLE     : boolean := true;
  constant DEV_FAST_DATA_XFER   : boolean := C_INCLUDE_BURST_CACHELN_SUPPORT /= 0; 
  constant DEV_BURST_PAGE_SIZE  : integer := 2**(27);
  constant DEV_MAX_BURST_SIZE   : integer := DEV_BURST_PAGE_SIZE;

  -- no interrupts
  constant IP_INTR_MODE_ARRAY   : INTEGER_ARRAY_TYPE := (0,0);
  constant IP_NUM_INTR          : integer := IP_INTR_MODE_ARRAY'length;
  constant ZERO_INTREVENT       : std_logic_vector(0 to IP_NUM_INTR-1) := (others => '0');
  constant INCLUDE_DEV_ISC      : BOOLEAN := false;
  constant INCLUDE_DEV_PENCODER : BOOLEAN := false;

  -- no master, slave only
  constant IP_MASTER_PRESENT    : BOOLEAN := false;

  -- zero constants for unused IPIF inputs
  constant ZERO_ADDR      : std_logic_vector(0 to C_PLB_AWIDTH-1)   := (others => '0');
  constant ZERO_DATA      : std_logic_vector(0 to C_PLB_DWIDTH-1)   := (others => '0');
  constant ZERO_BE        : std_logic_vector(0 to C_PLB_DWIDTH/8-1) := (others => '0');

  -- zero read fifo data input
  constant RDFIFO_DWIDTH   : integer := C_PLB_DWIDTH;
  constant ZERO_RFIFO_DATA : std_logic_vector(0 to RDFIFO_DWIDTH-1) := (others => '0');                                                      

  component PLB2DDR_WR
    port (
      Bus2IP_UserClk   : in  std_logic;
      Bus2IP_UserRst   : in  std_logic;
      Bus2IP_Data      : in  std_logic_vector(0 to 63);
      Bus2IP_Addr      : in  std_logic_vector(0 to 31);
      Bus2IP_BE        : in  std_logic_vector(0 to 7);
      Bus2IP_Burst     : in  std_logic;
      Bus2IP_CE        : in  std_logic_vector(0 to 0);
      Bus2IP_CS        : in  std_logic_vector(0 to 0);
      Bus2IP_RNW       : in  std_logic;
      Bus2IP_WrCE      : in  std_logic_vector(0 to 0);
      Bus2IP_WrReq     : in  std_logic;
      IP2Bus_WrAddrAck : out std_logic;
      IP2Bus_WrAck     : out std_logic;
      WR_DATA          : out std_logic_vector(71 downto 0);
      WR_ADDR          : out std_logic_vector(24 downto 0);
      WR_ADDR_WEN      : out std_logic;
      WR_ADDR_FF       : in  std_logic;
      WR_ADDR_AF       : in  std_logic;
      WR_RISE_WEN      : out std_logic;
      WR_RISE_FF       : in  std_logic;
      WR_RISE_AF       : in  std_logic;
      WR_FALL_WEN      : out std_logic;
      WR_FALL_FF       : in  std_logic;
      WR_FALL_AF       : in  std_logic;
      WR_DM_WEN        : out std_logic;
      WR_DM_FF         : in  std_logic;
      WR_DM_AF         : in  std_logic
    );
  end component;

  component PLB2DDR_RD
    port (
      Bus2IP_UserClk   : in  std_logic;
      Bus2IP_UserRst   : in  std_logic;
      Bus2IP_Addr      : in  std_logic_vector(0 to 31);
      Bus2IP_BE        : in  std_logic_vector(0 to 7);
      Bus2IP_Burst     : in  std_logic;
      Bus2IP_CE        : in  std_logic_vector(0 to 0);
      Bus2IP_CS        : in  std_logic_vector(0 to 0);
      Bus2IP_RNW       : in  std_logic;
      Bus2IP_RdCE      : in  std_logic_vector(0 to 0);
      Bus2IP_RdReq     : in  std_logic;
      IP2Bus_Data      : out std_logic_vector(0 to 63);
      IP2Bus_RdAddrAck : out std_logic;
      IP2Bus_RdAck     : out std_logic;
      RD_DATA          : in  std_logic_vector(63 downto 0);
      RD_ADDR          : out std_logic_vector(24 downto 0);
      RD_ADDR_WEN      : out std_logic;
      RD_ADDR_FF       : in  std_logic;
      RD_ADDR_AF       : in  std_logic;
      RD_RISE_REN      : out std_logic;
      RD_RISE_EF       : in  std_logic;
      RD_RISE_AE       : in  std_logic;
      RD_FALL_REN      : out std_logic;
      RD_FALL_EF       : in  std_logic;
      RD_FALL_AE       : in  std_logic
    );
  end component;

  signal Bus2IP_Addr      : std_logic_vector(0 to C_PLB_AWIDTH - 1);
  signal Bus2IP_BE        : std_logic_vector(0 to (C_PLB_DWIDTH/8) - 1);
  signal Bus2IP_Data      : std_logic_vector(0 to C_PLB_DWIDTH - 1);
  signal Bus2IP_RNW       : std_logic;
  signal Bus2IP_Burst     : std_logic;
  signal Bus2IP_RdReq     : std_logic;
  signal Bus2IP_WrReq     : std_logic;
  signal Bus2IP_CS        : std_logic_vector(0 to 0);
  signal Bus2IP_CE        : std_logic_vector(0 to 0);
  signal Bus2IP_RdCE      : std_logic_vector(0 to 0);
  signal Bus2IP_WrCE      : std_logic_vector(0 to 0);

  signal IP2Bus_Data      : std_logic_vector(0 to C_PLB_DWIDTH - 1);
  signal IP2Bus_AddrAck   : std_logic;
  signal IP2Bus_RdAck     : std_logic;
  signal IP2Bus_WrAck     : std_logic;
  signal IP2Bus_Error     : std_logic;
  signal IP2Bus_Retry     : std_logic;
  signal IP2Bus_ToutSup   : std_logic;
  signal IP2Bus_Busy      : std_logic;

  signal Bus2IP_UserClk   : std_logic;
  signal Bus2IP_UserRst   : std_logic;

  signal IP2Bus_WrAddrAck : std_logic;
  signal IP2Bus_RdAddrAck : std_logic;

  signal DDR_USER_CLK     : std_logic;
  signal DDR_USER_RST     : std_logic;

  signal DDR_WR_DATA      : std_logic_vector(71 downto 0);
  signal DDR_WR_ADDR      : std_logic_vector(24 downto 0);
  signal DDR_WR_ADDR_WEN  : std_logic;
  signal DDR_WR_ADDR_FF   : std_logic;
  signal DDR_WR_ADDR_AF   : std_logic;
  signal DDR_WR_RISE_WEN  : std_logic;
  signal DDR_WR_RISE_FF   : std_logic;
  signal DDR_WR_RISE_AF   : std_logic;
  signal DDR_WR_FALL_WEN  : std_logic;
  signal DDR_WR_FALL_FF   : std_logic;
  signal DDR_WR_FALL_AF   : std_logic;
  signal DDR_WR_DM_WEN    : std_logic;
  signal DDR_WR_DM_FF     : std_logic;
  signal DDR_WR_DM_AF     : std_logic;

  signal DDR_RD_DATA      : std_logic_vector(63 downto 0);
  signal DDR_RD_ADDR      : std_logic_vector(24 downto 0);
  signal DDR_RD_ADDR_WEN  : std_logic;
  signal DDR_RD_ADDR_FF   : std_logic;
  signal DDR_RD_ADDR_AF   : std_logic;
  signal DDR_RD_RISE_REN  : std_logic;
  signal DDR_RD_RISE_EF   : std_logic;
  signal DDR_RD_RISE_AE   : std_logic;
  signal DDR_RD_FALL_REN  : std_logic;
  signal DDR_RD_FALL_EF   : std_logic;
  signal DDR_RD_FALL_AE   : std_logic;

begin

I_PLB_IPIF: entity plb_ipif_v1_00_f.plb_ipif
  generic map (
    C_ARD_ID_ARRAY         => ARD_ID_ARRAY,
    C_ARD_ADDR_RANGE_ARRAY => ARD_ADDR_RANGE_ARRAY,
    C_ARD_DWIDTH_ARRAY     => ARD_DWIDTH_ARRAY,
    C_ARD_NUM_CE_ARRAY     => ARD_NUM_CE_ARRAY,
    C_DEV_BLK_ID           => DEV_BLK_ID,
    C_DEV_MIR_ENABLE       => DEV_MIR_ENABLE,
    C_DEV_BURST_ENABLE     => DEV_BURST_ENABLE,
    C_DEV_FAST_DATA_XFER   => DEV_FAST_DATA_XFER,
    C_DEV_BURST_PAGE_SIZE  => DEV_BURST_PAGE_SIZE,
    C_DEV_MAX_BURST_SIZE   => DEV_MAX_BURST_SIZE,
    C_DEV_DPHASE_TIMEOUT   => DEV_DPHASE_TIMEOUT,
    C_INCLUDE_DEV_ISC      => INCLUDE_DEV_ISC,
    C_INCLUDE_DEV_PENCODER => INCLUDE_DEV_PENCODER,
    C_IP_INTR_MODE_ARRAY   => IP_INTR_MODE_ARRAY,
    C_IP_MASTER_PRESENT    => IP_MASTER_PRESENT,
    C_PLB_MID_WIDTH        => C_PLB_MID_WIDTH,
    C_PLB_NUM_MASTERS      => C_PLB_NUM_MASTERS,
    C_PLB_AWIDTH           => C_PLB_AWIDTH,
    C_PLB_DWIDTH           => C_PLB_DWIDTH,
    C_PLB_CLK_PERIOD_PS    => C_PLB_CLK_PERIOD_PS,
    C_IPIF_DWIDTH          => C_PLB_DWIDTH,
    C_IPIF_AWIDTH          => C_PLB_AWIDTH,
    C_FAMILY               => C_FAMILY )
  port map (
    -- System signals
    PLB_clk               => PLB_Clk,
    Reset                 => PLB_Rst,
    Freeze                => '0',
    IP2INTC_Irpt          => open,
    -- Bus Slave signals
    PLB_ABus              => PLB_ABus,
    PLB_PAValid           => PLB_PAValid,
    PLB_SAValid           => PLB_SAValid,
    PLB_rdPrim            => PLB_rdPrim,
    PLB_wrPrim            => PLB_wrPrim,
    PLB_masterID          => PLB_masterID,
    PLB_abort             => PLB_abort,
    PLB_busLock           => PLB_busLock,
    PLB_RNW               => PLB_RNW,
    PLB_BE                => PLB_BE,
    PLB_MSize             => PLB_MSize,
    PLB_size              => PLB_size,
    PLB_type              => PLB_type,
    PLB_compress          => PLB_compress,
    PLB_guarded           => PLB_guarded,
    PLB_ordered           => PLB_ordered,
    PLB_lockErr           => PLB_lockErr,
    PLB_wrDBus            => PLB_wrDBus,
    PLB_wrBurst           => PLB_wrBurst,
    PLB_rdBurst           => PLB_rdBurst,
    PLB_pendReq           => PLB_pendReq,
    PLB_pendPri           => PLB_pendPri,
    PLB_reqPri            => PLB_reqPri,
    Sl_addrAck            => Sl_addrAck,
    Sl_SSize              => Sl_SSize,
    Sl_wait               => Sl_wait,
    Sl_rearbitrate        => Sl_rearbitrate,
    Sl_wrDAck             => Sl_wrDAck,
    Sl_wrComp             => Sl_wrComp,
    Sl_wrBTerm            => Sl_wrBTerm,
    Sl_rdDBus             => Sl_rdDBus,
    Sl_rdWdAddr           => Sl_rdWdAddr,
    Sl_rdDAck             => Sl_rdDAck,
    Sl_rdComp             => Sl_rdComp,
    Sl_rdBTerm            => Sl_rdBTerm,
    Sl_MBusy              => Sl_MBusy,
    Sl_MErr               => Sl_MErr,
    -- Bus Master Signals
    PLB_MAddrAck          => '0',
    PLB_MSSize            => "00",
    PLB_MRearbitrate      => '0',
    PLB_MBusy             => '0',
    PLB_MErr              => '0',
    PLB_MWrDAck           => '0',
    PLB_MRdDBus           => ZERO_DATA,
    PLB_MRdWdAddr         => "0000",
    PLB_MRdDAck           => '0',
    PLB_MRdBTerm          => '0',
    PLB_MWrBTerm          => '0',
    M_request             => open,
    M_priority            => open,
    M_buslock             => open,
    M_RNW                 => open,
    M_BE                  => open,
    M_MSize               => open,
    M_size                => open,
    M_type                => open,
    M_compress            => open,
    M_guarded             => open,
    M_ordered             => open,
    M_lockErr             => open,
    M_abort               => open,
    M_ABus                => open,
    M_wrDBus              => open,
    M_wrBurst             => open,
    M_rdBurst             => open,
    -- IP Interconnect (IPIC) port signals
    --System Signals
    IP2Bus_Clk            => '0',
    Bus2IP_Clk            => open,
    Bus2IP_Reset          => open,
    Bus2IP_Freeze         => open,
    -- IP Slave signals
    IP2Bus_IntrEvent      => ZERO_INTREVENT,
    IP2Bus_Data           => IP2Bus_Data,
    IP2Bus_WrAck          => IP2Bus_WrAck,
    IP2Bus_RdAck          => IP2Bus_RdAck,
    IP2Bus_Retry          => IP2Bus_Retry,
    IP2Bus_Error          => IP2Bus_Error,
    IP2Bus_ToutSup        => IP2Bus_ToutSup,
    IP2Bus_PostedWrInh    => '0',
    IP2Bus_Busy           => IP2Bus_Busy,
    IP2Bus_AddrAck        => IP2Bus_AddrAck,
    IP2Bus_BTerm          => '0',
    Bus2IP_Addr           => Bus2IP_Addr,
    Bus2IP_Data           => Bus2IP_Data,
    Bus2IP_RNW            => Bus2IP_RNW,
    Bus2IP_BE             => Bus2IP_BE,
    Bus2IP_Burst          => Bus2IP_Burst,
    Bus2IP_IBurst         => open,
    Bus2IP_WrReq          => Bus2IP_WrReq,
    Bus2IP_RdReq          => Bus2IP_RdReq,
    Bus2IP_RNW_Early      => open,
    Bus2IP_PselHit        => open,
    Bus2IP_CS             => Bus2IP_CS,
    Bus2IP_CE             => Bus2IP_CE,
    Bus2IP_RdCE           => Bus2IP_RdCE,
    Bus2IP_WrCE           => Bus2IP_WrCE,
    -- IP to DMA Support Signals (Length and Status FIFO I/O)
    IP2DMA_RxLength_Empty => '0',
    IP2DMA_RxStatus_Empty => '0',
    IP2DMA_TxLength_Full  => '0',
    IP2DMA_TxStatus_Empty => '0',
    -- IP Master Signals
    IP2Bus_Addr           => ZERO_ADDR,
    IP2Bus_MstBE          => ZERO_BE,
    IP2IP_Addr            => ZERO_ADDR,
    IP2Bus_MstWrReq       => '0',
    IP2Bus_MstRdReq       => '0',
    IP2Bus_MstBurst       => '0',
    IP2Bus_MstBusLock     => '0',
    Bus2IP_MstWrAck       => open,
    Bus2IP_MstRdAck       => open,
    Bus2IP_MstRetry       => open,
    Bus2IP_MstError       => open,
    Bus2IP_MstTimeOut     => open,
    Bus2IP_MstLastAck     => open,
    -- RdPFIFO Signals
    IP2RFIFO_WrReq        => '0',
    IP2RFIFO_Data         => ZERO_RFIFO_DATA,
    IP2RFIFO_WrMark       => '0',
    IP2RFIFO_WrRelease    => '0',
    IP2RFIFO_WrRestore    => '0',
    RFIFO2IP_WrAck        => open,
    RFIFO2IP_AlmostFull   => open,
    RFIFO2IP_Full         => open,
    RFIFO2IP_Vacancy      => open,
    -- WrPFIFO signals
    IP2WFIFO_RdReq        => '0',
    IP2WFIFO_RdMark       => '0',
    IP2WFIFO_RdRelease    => '0',
    IP2WFIFO_RdRestore    => '0',
    WFIFO2IP_Data         => open,
    WFIFO2IP_RdAck        => open,
    WFIFO2IP_AlmostEmpty  => open,
    WFIFO2IP_Empty        => open,
    WFIFO2IP_Occupancy    => open,
    -- IP DMA signals
    IP2Bus_DMA_Req        => '0',
    Bus2IP_DMA_Ack        => open );

  Bus2IP_UserClk <= PLB_Clk;
  Bus2IP_UserRst <= PLB_Rst;

  DDR_USER_CLK <= Bus2IP_UserClk;
  DDR_USER_RST <= Bus2IP_UserRst;

  IP2Bus_Busy    <= '0';
  IP2Bus_Retry   <= '0'; -- no retry
  IP2Bus_Error   <= '0'; -- no error
  IP2Bus_ToutSup <= '0'; -- no timeout suppress. If IP2Bus_RdAck or IP2Bus_WrAck is delayed more than 64 clocks,
                         -- drive IP2Bus_ToutSup to '1' to avoid data phase timeout in IPIF.

  IP2Bus_AddrAck <= IP2Bus_WrAddrAck OR IP2Bus_RdAddrAck;

  iPLB2DDR_WR : PLB2DDR_WR
    port map (
      Bus2IP_UserClk   => Bus2IP_UserClk,
      Bus2IP_UserRst   => Bus2IP_UserRst,
      Bus2IP_Data      => Bus2IP_Data,
      Bus2IP_Addr      => Bus2IP_Addr,
      Bus2IP_BE        => Bus2IP_BE,
      Bus2IP_Burst     => Bus2IP_Burst,
      Bus2IP_CE        => Bus2IP_CE,
      Bus2IP_CS        => Bus2IP_CS,
      Bus2IP_RNW       => Bus2IP_RNW,
      Bus2IP_WrCE      => Bus2IP_WrCE,
      Bus2IP_WrReq     => Bus2IP_WrReq,
      IP2Bus_WrAddrAck => IP2Bus_WrAddrAck,
      IP2Bus_WrAck     => IP2Bus_WrAck,
      WR_DATA          => DDR_WR_DATA,
      WR_ADDR          => DDR_WR_ADDR,
      WR_ADDR_WEN      => DDR_WR_ADDR_WEN,
      WR_ADDR_FF       => DDR_WR_ADDR_FF,
      WR_ADDR_AF       => DDR_WR_ADDR_AF,
      WR_RISE_WEN      => DDR_WR_RISE_WEN,
      WR_RISE_FF       => DDR_WR_RISE_FF,
      WR_RISE_AF       => DDR_WR_RISE_AF,
      WR_FALL_WEN      => DDR_WR_FALL_WEN,
      WR_FALL_FF       => DDR_WR_FALL_FF,
      WR_FALL_AF       => DDR_WR_FALL_AF,
      WR_DM_WEN        => DDR_WR_DM_WEN,
      WR_DM_FF         => DDR_WR_DM_FF,
      WR_DM_AF         => DDR_WR_DM_AF );

  iPLB2DDR_RD : PLB2DDR_RD
    port map (
      Bus2IP_UserClk   => Bus2IP_UserClk,
      Bus2IP_UserRst   => Bus2IP_UserRst,
      Bus2IP_Addr      => Bus2IP_Addr,
      Bus2IP_BE        => Bus2IP_BE,
      Bus2IP_Burst     => Bus2IP_Burst,
      Bus2IP_CE        => Bus2IP_CE,
      Bus2IP_CS        => Bus2IP_CS,
      Bus2IP_RNW       => Bus2IP_RNW,
      Bus2IP_RdCE      => Bus2IP_RdCE,
      Bus2IP_RdReq     => Bus2IP_RdReq,
      IP2Bus_Data      => IP2Bus_Data,
      IP2Bus_RdAddrAck => IP2Bus_RdAddrAck,
      IP2Bus_RdAck     => IP2Bus_RdAck,
      RD_DATA          => DDR_RD_DATA,
      RD_ADDR          => DDR_RD_ADDR,
      RD_ADDR_WEN      => DDR_RD_ADDR_WEN,
      RD_ADDR_FF       => DDR_RD_ADDR_FF,
      RD_ADDR_AF       => DDR_RD_ADDR_AF,
      RD_RISE_REN      => DDR_RD_RISE_REN,
      RD_RISE_EF       => DDR_RD_RISE_EF,
      RD_RISE_AE       => DDR_RD_RISE_AE,
      RD_FALL_REN      => DDR_RD_FALL_REN,
      RD_FALL_EF       => DDR_RD_FALL_EF,
      RD_FALL_AE       => DDR_RD_FALL_AE );

  PPC_TO_DDR_IF(71  downto  0) <= DDR_WR_DATA;
  PPC_TO_DDR_IF(96  downto 72) <= DDR_WR_ADDR;
  PPC_TO_DDR_IF(121 downto 97) <= DDR_RD_ADDR;

  PPC_TO_DDR_IF(122) <= DDR_USER_CLK;
  PPC_TO_DDR_IF(123) <= DDR_USER_RST;
  PPC_TO_DDR_IF(124) <= DDR_WR_ADDR_WEN;
  PPC_TO_DDR_IF(125) <= DDR_WR_RISE_WEN;
  PPC_TO_DDR_IF(126) <= DDR_WR_FALL_WEN;
  PPC_TO_DDR_IF(127) <= DDR_WR_DM_WEN;
  PPC_TO_DDR_IF(128) <= DDR_RD_ADDR_WEN;
  PPC_TO_DDR_IF(129) <= DDR_RD_RISE_REN;
  PPC_TO_DDR_IF(130) <= DDR_RD_FALL_REN;

  DDR_RD_DATA <= DDR_TO_PPC_IF(63 downto 0);

  DDR_WR_ADDR_FF <= DDR_TO_PPC_IF(64);
  DDR_WR_ADDR_AF <= DDR_TO_PPC_IF(65);
  DDR_WR_RISE_FF <= DDR_TO_PPC_IF(66);
  DDR_WR_RISE_AF <= DDR_TO_PPC_IF(67);
  DDR_WR_FALL_FF <= DDR_TO_PPC_IF(68);
  DDR_WR_FALL_AF <= DDR_TO_PPC_IF(69);
  DDR_WR_DM_FF   <= DDR_TO_PPC_IF(70);
  DDR_WR_DM_AF   <= DDR_TO_PPC_IF(71);
  DDR_RD_ADDR_FF <= DDR_TO_PPC_IF(72);
  DDR_RD_ADDR_AF <= DDR_TO_PPC_IF(73);
  DDR_RD_RISE_EF <= DDR_TO_PPC_IF(74);
  DDR_RD_RISE_AE <= DDR_TO_PPC_IF(75);
  DDR_RD_FALL_EF <= DDR_TO_PPC_IF(76);
  DDR_RD_FALL_AE <= DDR_TO_PPC_IF(77);

end implementation;
